﻿import os
import signal
import socket 
from contextlib import closing

import app.util
import app.servicecontrol
import app.batch
import app.defaults
import app.logutil

class BasePortAvailabilityCheckServiceAction(app.servicecontrol.ControllerAction):
    def __init__(self, services, logger):
        self.services = services
        return super(BasePortAvailabilityCheckServiceAction, self).__init__(logger)

    def isListening(self, host, port):
        socketTimeout = 20

        def alarmHandler(signum,frame):
            raise ConnectionTimeoutException("Connection timeout")

        self.logger.info("Checking endpoint %s:%d..." % (host, port))
        try:
            # Install signal handler to avoid connect() from blocking indefinitely
            signal.signal(signal.SIGALRM, alarmHandler)
            signal.alarm(socketTimeout)

            with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
                errno = sock.connect_ex((host, port))
                if errno == 0:
                    self.logger.info("Endpoint %s:%d is available" % (host, port))
                    return True
                else:
                    self.logger.error("Endpoint %s:%d not available: %d" % (host, port, errno))
                    return False
            
        except ConnectionTimeoutException:
            self.logger.error("Connectingt to endpoint %s:%d timed out" % (host, port))
            return False
        except Exception, sockErrMsg:
            if socket.herror:
                self.logger.error("Host not found %s" % host)
            else:
                self.logger.error("Connecting to endpoint %s:%d is failed and error is %s" %
                        (host, port, sockErrMsg))
            return False
        finally:
            signal.alarm(0)

    def chain(self, portAvailabilityCheck):
        assert isinstance(portAvailabilityCheck, PortAvailabilityCheckServiceAction)

        return PortAvailabilityCheckServiceAction(
            self.services + portAvailabilityCheck.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.services
    
    def execute(self):
        raise Exception("this class is abstract")

class ProvidedEndpointAvailableCheckServiceAction(BasePortAvailabilityCheckServiceAction):
    def execute(self):
        allOk = True

        for service in self.services:
            """ Check if all ports the service intends to use are available """
            for endpoint in service.getProvidedEndpoints():
                if endpoint.isViaOtherPort():
                    # Ignore this as the port is owned by a different service.
                    pass
                else:
                    # To be extra thorugh, we check both 'localhost' and the 
                    # physical machine name.
                    if self.isListening("localhost", endpoint.getPort()):
                        self.printLog("error", "Port %d in use by different service" % 
                                      (endpoint.getPort()))
                        allOk = False
                    elif self.isListening(endpoint.getPhysicalHost(), endpoint.getPort()):
                        self.printLog("error", "Port %d on %s in use by different service" % 
                                      (endpoint.getPort(), service.getName()))
                        allOk = False

        return allOk

class ProvidedEndpointListeningCheckServiceAction(BasePortAvailabilityCheckServiceAction):
    def execute(self):
        allOk = True

        for service in self.services:
            """ Check if all ports the service intends to use are available """
            for endpoint in service.getProvidedEndpoints():
                if endpoint.isViaOtherPort():
                    # Ignore this as the port is owned by a different service.
                    pass
                else:
                    # Only check 'localhost' as we do not really know which IP
                    # the service is supposed to bind to.
                    if not self.isListening("localhost", endpoint.getPort()):
                        self.printLog("error", "Port %d on localhost not listening" % 
                                      (endpoint.getPort()))
                        allOk = False

        return allOk

class RequiredEndpointListeningCheckServiceAction(BasePortAvailabilityCheckServiceAction):
    def execute(self):
        allOk = True

        for service in self.services:
            """ Check if all ports the service intends to use are available """
            for reqEndpoint in service.getRequiredEndpoints():
                if not reqEndpoint.isRequiredForStartup():
                    # Nevermind this one.
                    continue

                endpoint = reqEndpoint.getProviderEndpoint()

                if not self.isListening(endpoint.getHost(), endpoint.getPort()):
                    self.printLog("error", "Port %d on %s needed by %s not listening" % 
                                    (endpoint.getPort(), endpoint.getPhysicalHost(), service.getName()))
                    allOk = False

        return allOk

class EnvironmentVerificationCheckAction(app.servicecontrol.ControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        return super(EnvironmentVerificationCheckAction, self).__init__(logger)
    
    def chain(self, envVerificationAction):
        assert isinstance(envVerificationAction, EnvironmentVerificationCheckAction)

        return EnvironmentVerificationCheckAction(
            self.__services + envVerificationAction.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def execute(self):
        allOk = True

        for service in self.__services:
            # A separate batch is run for each service so that we the order
            # corresponds to the order of the services list

            scriptsFolder = os.path.join(
                app.defaults.DEFAULT_SERVICE_DIR, 
                service.getName(),
                "hooks",
                "verification")

            if os.path.isdir(scriptsFolder):
                self.printLog('title', "Verifying environment for %s..." % service.getName())

                batch = app.batch.VerificationBatch(
                    self.logger, 
                    app.landscape.Landscape.getSiteOptions())
                batch.addFolder(scriptsFolder)

                try:
                    batch.execute()
                except Exception, err:
                    self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                    allOk = False
            
        if not allOk:
            self.printLog("error", "\nVerification failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))

        return allOk

class ConnectionTimeoutException(Exception):
    pass
