﻿import os
import shutil
import app.defaults
import app.util

class JavaCertStore(object):
    ALIAS = "node1"

    def __init__(self, keystorePath):
        self.__keystorePath = keystorePath

    @staticmethod
    def generateSelfSignedCertificate(keystorePath, commonName, validityInDays):
         javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-genkeypair",
            "-keystore", keystorePath,
            "-keyalg", "RSA",
            "-alias", JavaCertStore.ALIAS,
            "-dname", commonName,
            "-storepass", "changeit", 
            "-keypass", "changeit",
            "-validity", str(validityInDays)
         ]

         app.util.executeCommand(javaCommand, useShell=False)

         return JavaCertStore(keystorePath)

    def exportCertificateToDerFile(self, path):
        """ Export in DER format """
        
        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-exportcert",
            "-keystore", self.__keystorePath,
            "-alias", JavaCertStore.ALIAS,
            "-storepass", "changeit", 
            "-file", path
        ]

        app.util.executeCommand(javaCommand, useShell=False)

    def exportCertificateToPemFile(self, path):
        """ Export in PEM format """
        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-exportcert",
            "-keystore", self.__keystorePath,
            "-alias", JavaCertStore.ALIAS,
            "-storepass", "changeit", 
            "-file", path,
            "-rfc"
        ]

        app.util.executeCommand(javaCommand, useShell=False)

class JavaTrustStore(object):
    DEFAULT_PASSWORD = "changeit"

    def __init__(self, path=None):
        self.__path = path

        defaultPaths = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, "jre/lib/security/jssecacerts"),
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, "jre/lib/security/cacerts")
        ]

        if not self.__path:
            for path in defaultPaths:
                if os.path.isfile(path):
                    self.__path = path

        if not self.__path:
            raise Exception("No path specified and no default store found on system")

    def getPath(self):
        return self.__path

    @staticmethod
    def createTrustStore(path, caRootPemFile):
        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-importcert", 
            "-keystore", path, 
            "-alias", "CA-cert",
            "-storepass", JavaTrustStore.DEFAULT_PASSWORD,
            "-file", caRootPemFile,
            "-noprompt"
            ]

        app.util.executeCommand(javaCommand, useShell=False)

        return JavaTrustStore(path)

    def listTrustedOwners(self):
        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-list",
            "-v",
            "-keystore", self.__path, 
            "-storepass", JavaTrustStore.DEFAULT_PASSWORD,
            ]

        output = app.util.executeCommand(" ".join(javaCommand) + "|grep Owner|cut -d : -f 2").split("\n")
        return [line.strip() for line in output if line]

    def importCertificateFromServer(self, host, port):
        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'bin/java'),
            "-cp", app.defaults.CTFUTIL_CLASSPATH,
            "com.vasoftware.sf.system.utils.JSSEInstallCertificate",
            "%s:%d" % (host, port)
            ]
            
        if self.__path:
            javaCommand.insert(1, "-Dcertstore=" + self.__path)

        app.util.executeCommand(javaCommand, useShell=False)

    def deleteCertificate(self, alias):
        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-delete",
            "-alias", alias,
            "-keystore", self.__path, 
            "-storepass", JavaTrustStore.DEFAULT_PASSWORD,
            "-noprompt"
            ]

        app.util.executeCommand(javaCommand, useShell=False)

    def importCertificateFile(self, alias, pemCertPath):
        # Importing fails if there if a certificate with the same alias is already
        # available. Therefore, delete first.
        try:
            self.deleteCertificate(alias)
        except:
            pass

        javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'jre/bin/keytool'),
            "-import",
            "-alias", alias,
            "-keystore", self.__path, 
            "-storepass", JavaTrustStore.DEFAULT_PASSWORD,
            "-file", pemCertPath,
            "-noprompt"
            ]

        app.util.executeCommand(javaCommand, useShell=False)

    def importCertificateChainFile(self, alias, pemChainFile):
        self.importCertificateFile(alias, pemChainFile)

class LinuxTrustStore(object):
    ANCHORS_DIR = "/etc/pki/ca-trust/source/anchors/"

    def __updateTrustStore(self):
        app.util.executeCommand(["update-ca-trust", "extract"])

    def importCertificateFile(self, alias, pemCertPath):
        shutil.copy(pemCertPath, os.path.join(LinuxTrustStore.ANCHORS_DIR, alias + ".pem"))
        self.__updateTrustStore()

    def importCertificateChainFile(self, alias, pemChainFile):
        self.importCertificateFile(alias, pemChainFile)
