import os
import sys
import socket
import app.options
import app.servicecontrol
import app.servicemanifest

# Object Model
#
#                                        +-------------+ *   1 +--------------+
#                                        |Prov. Endpnt |------>|Req. Endpnt   |
#                                        +-------------+       +--------------+
#                                               * ^-----+      * +-^
#                                                       |        |
#                                                       | 1    1 |
#  +------------+ 1     * +-------------+ 1        * +--------------+
#  |  Site      +-------->+  Machine    +----------->+   Service    |
#  +------------+         +-------------+            +--------------+
# 

class ProvidedEndpoint(object):

    def __init__(self, id, scope, machine, port, viaInfo = None):
        self.__id = id
        self.__scope = scope
        self.__port = port
        self.__machine = machine
        self.__viaInfo = viaInfo
        self.__clientServices = []

    def isViaOtherPort(self):
        """ Checks if this endpoint is to be accessed via a different port """

        return self.__viaInfo != None

    def __getViaPort(self): 
        """ Get port via which this endpoint is to be accessed """

        viaService = self.__machine.getService(self.__viaInfo.getService())
        return viaService.getProvidedEndpoint(self.__viaInfo.getId()).getPort()

    def __repr__(self):
        if self.isViaOtherPort():
            viaInfo = " (via %s)" % self.__viaInfo.getService()
        else:
            viaInfo = ""

        return "[%s] (%s) on %s:%d%s" % (self.getId(), self.getScope(), self.getHost(), self.getPort(), viaInfo)

    def getId(self):
        return self.__id
    
    def getScope(self):
        return self.__scope
    
    def getPort(self):
        if self.isViaOtherPort():
            return self.__getViaPort()
        else:
            return self.__port

    def getMachine(self):
        return self.__machine

    def getPhysicalHost(self):
        return self.__machine.getPhysicalName()
    
    def getPublicHost(self):
        if self.__scope != app.servicemanifest.PROVIDED_ENDPOINT_SCOPE_PUBLIC:
            raise ConfigurationException("endpoint is not public")

        return self.__machine.getPublicName()
    
    def getLogicalHost(self):
        # Logical hosts (load balancers) are not supported yet, so return
        # the physical host for now.
        return self.getPhysicalHost()

    def getHost(self, preferLocalhost=True):
        """ Return the 'right' host to use """

        if preferLocalhost and self.__machine.isLocal():
            # Always prefer 'localhost'
            return "localhost"
        elif self.__scope == app.servicemanifest.PROVIDED_ENDPOINT_SCOPE_PUBLIC:
            # Remote box, go thru public endpoint/load balancer.
            return self.getPublicHost()
        else:
            # Internal cross-server communication.
            return self.getPhysicalHost()

    def getViaServiceName(self):
        if not self.__viaInfo:
            raise ConfigurationExceptio("endpoint does not use via")

        return self.__viaInfo.getService()

    def snapClient(self, clientService):
        self.__clientServices.append(clientService)

    def getClients(self):
        return self.__clientServices

class RequiredEndpoint(object):
    def __init__(self, id, service, requiredForDeployment, requiredForStartup):
        self.__id = id
        self.__service = service
        self.__requiredForDeployment = requiredForDeployment
        self.__requiredForStartup = requiredForStartup
        self.__snappedEndpoint = None
        self.__providerService = None

    def __repr__(self):
        if self.__snappedEndpoint:
            snapInfo = " -> %s" % self.__snappedEndpoint
        else:
            snapInfo = ""

        return "[%s] of %s%s" % (self.getId(), self.getService(), snapInfo)

    def getId(self):
        return self.__id
    
    def getService(self):
        return self.__service

    def isRequiredForStartup(self):
        return self.__requiredForStartup

    def isRequiredForDeployment(self):
        return self.__requiredForDeployment

    def snapProvider(self, providerService, providedEndpoint):
        self.__providerService = providerService
        self.__snappedEndpoint = providedEndpoint

    def getProviderService(self):
        return self.__providerService

    def getProviderEndpoint(self):
        return self.__snappedEndpoint

class DaemonStartupInfo(object):
    def __init__(self, isExternal, user=None, group=None, startupTimeout=None):
        self.__isExternal = isExternal
        self.__user = user
        self.__group = group
        self.__startupTimeout = startupTimeout

    def isExternal(self):
        return self.__isExternal

    def getUser(self):
        return self.__user

    def getGroup(self):
        return self.__group

    def getStartupTimeout(self):
        return self.__startupTimeout

class Service(object):
    def __init__(self, name, daemonStartupInfo, machine, providedEndpoints, requiredEndpoints, integrityConfFiles, isAlias=False):
        self.__name = name
        self.__daemonStartupInfo = daemonStartupInfo
        self.__machine = machine
        self.__providedEndpoints = providedEndpoints
        self.__requiredEndpoints = requiredEndpoints
        self.__dependenciesInDeploymentOrder = []
        self.__transitiveDependenciesInDeploymentOrder = []

        self.__containerService = None
        self.__containedServices = []
        self.__integrityConfFiles = integrityConfFiles
        self.__isAlias = isAlias

    def __repr__(self):
        return self.__name

    def getName(self):
        return self.__name

    def getMachine(self):
        return self.__machine

    def isAlias(self):
        return self.__isAlias

    def getProvidedEndpoints(self):
        return self.__providedEndpoints

    def getRequiredEndpoints(self):
        return self.__requiredEndpoints

    def getIntegrityConfFiles(self):
        return self.__integrityConfFiles

    def getProvidedEndpoint(self, id, scope=None):
        for endpoint in self.__providedEndpoints:
            if endpoint.getId() == id and (scope == None or scope == endpoint.getScope()):
                return endpoint

        return None

    def getRequiredEndpoint(self, id, service=None):
        for endpoint in self.__requiredEndpoints:
            if endpoint.getId() == id and (service == None or service == endpoint.getService()):
                return endpoint

        return None

    def setDeploymentDependencies(self, dependenciesInDeploymentOrder, transitiveDependenciesInDeploymentOrder, containerService):
        self.__dependenciesInDeploymentOrder = dependenciesInDeploymentOrder
        self.__transitiveDependenciesInDeploymentOrder = transitiveDependenciesInDeploymentOrder
        
        if containerService:
            if self.isDaemon():
                raise ConfigurationException(
                    "Service %s is a daemon and thus must not define a container" % self.__name)

            self.__containerService = containerService
            containerService.addContainedService(self)

    def addContainedService(self, service):
        if not self in service.getDeploymentDependencies():
            raise ConfigurationException(
                "Service %s is hosted in container %s but it does not define a deployment dependency" % 
                (service.getName(), self.__name))
        self.__containedServices.append(service)

    def getContainingService(self):
        return self.__containerService

    def getContainedServices(self):
        return self.__containedServices

    def getDeploymentDependencies(self, transitive=False):
        if transitive:
            return self.__transitiveDependenciesInDeploymentOrder
        else:
            return self.__dependenciesInDeploymentOrder

    def getStartupDependencies(self, filteredByMachine = None):
        """ Returns the list of services that need to be available for this service to start up """

        dependencies = set()

        # Any vital endpoint counts as startup dependency
        for reqEndpoint in self.__requiredEndpoints:
            if reqEndpoint.isRequiredForStartup():
                if not reqEndpoint.getProviderEndpoint():
                    raise UnresolvedEndpointDependenciesException(
                        "Service %s requires %s of %s, but this endpoint has not been resolved yet" % 
                            (self.__name, reqEndpoint.getId(), reqEndpoint.getService()))

                if not filteredByMachine or filteredByMachine == reqEndpoint.getProviderEndpoint().getMachine():
                    service = reqEndpoint.getProviderService()
                    dependencies.add(service)

                    # If the service is contained by another service, consider this as a dependency too.
                    serviceContainer = service.getContainingService()
                    if serviceContainer:
                        dependencies.add(serviceContainer)
            
        # Any service via which we expose endpoints is a startup dependency.
        for provEndpoint in self.__providedEndpoints:
            if provEndpoint.isViaOtherPort():
                if self.__containerService and \
                   self.__containerService.getName() == provEndpoint.getViaServiceName():
                    # The container will start up first anyway - if we added a 
                    # startup dependency here, it would create a dependency cycle.
                    continue

                viaService = None
                for service in self.__dependenciesInDeploymentOrder:
                    if service.getName() == provEndpoint.getViaServiceName():
                        viaService = service
                        break

                if viaService:
                     dependencies.add(service)
                else:
                    raise UnresolvedEndpointDependenciesException(
                        "Service %s exposes endpoint via %s, but this service is not a deployment dependency" % 
                        (self.__name, provEndpoint.getViaServiceName()))

        # If this service serves as container for other services, we need to consider the
        # contained services dependencies as well.
        for containedService in self.__containedServices:
            dependencies.add(containedService)

        return dependencies

    def getMachineStartupDependencies(self):
        """ Returns the list of machines whose services need to be available for this service to start up """

        machines = set()

        # Any vital endpoint counts as startup dependency
        for reqEndpoint in self.__requiredEndpoints:
            if reqEndpoint.isRequiredForStartup():
                if not reqEndpoint.getProviderEndpoint():
                    raise UnresolvedEndpointDependenciesException(
                        "Service %s requires %s of %s, but this endpoint has not been resolved yet" % 
                            (self.__name, reqEndpoint.getId(), reqEndpoint.getService()))

                if reqEndpoint.isRequiredForStartup():
                    machines.add(reqEndpoint.getProviderEndpoint().getMachine())

        return machines

    def isDaemon(self):
        return self.__daemonStartupInfo != None

    def getDaemonStartupInfo(self):
        return self.__daemonStartupInfo

    def getController(self, logger):
        if not self.__machine.isLocal():
            raise ServiceException("Remote service cannot be controlled")

        return app.servicecontrol.ServiceController.forService(self, logger)

class Machine(object):
    def __init__(self, phyicalName, publicName, isLocal, services = [], deploymentOrder = []):
        if phyicalName == "localhost":
            # We need the proper name.
            phyicalName = Machine.getLocalHostName()

        self.__phyicalName = phyicalName
        self.__publicName = publicName
        self.__isLocal = isLocal
        self.__services = services
        self.__deploymentOrder = deploymentOrder
        
    def __repr__(self):
        return self.getPhysicalName()

    def __eq__(self, other):
        return isinstance(other, Machine) and other.__phyicalName == self.__phyicalName

    @staticmethod
    def getLocalHostName():
        return socket.gethostname()

    def getPhysicalName(self):
        return self.__phyicalName
    
    def getPublicName(self):
        return self.__publicName

    def getIp(self):
        try:
            return socket.gethostbyname(self.__phyicalName)
        except Exception, e:
            raise NetworkException("Could not resolve IP for machine '%s'" % self.__phyicalName)
    
    def isLocal(self):
        return self.__isLocal

    def setServices(self, services, deploymentOrder):
        self.__services = services
        self.__deploymentOrder = deploymentOrder

    def getServices(self):
        return self.__services

    def getServiceNames(self):
        return [service.getName() for service in self.__services]

    def hasService(self, serviceName):
        return serviceName in self.getServiceNames()

    def getService(self, serviceName):
        for service in self.__services:
            if service.getName() == serviceName:
                return service

        return None

    def getServicesInDeploymentOrder(self):
        return [self.getService(serviceName) 
                for serviceName in self.__deploymentOrder
                if self.hasService(serviceName)]

    def getServicesInAlphabeticOrder(self):
        return sorted(self.__services, key=lambda s : s.getName())
    
    def getDaemonServices(self):
        return [service for service in self.__services if service.isDaemon()]

    def getServicesInStartOrder(self):
        # Build a dict `service : set([dependencies...])`...
        dependencyGraph = dict((service, service.getStartupDependencies(self)) 
                               for service in self.getServices())

        # ...and sort it topologically. Finally, throw out anything that is not a daemon.
        import ext.toposort
        return [service for service in ext.toposort.toposort_flatten(dependencyGraph)]
    
    def getMachineStartupDependencies(self):
        machines = set()

        for service in self.__services:
            machines |= service.getMachineStartupDependencies()

        if self in machines:
            machines.remove(self)
        return machines

    def getController(self, logger):
        if not self.isLocal():
            raise ServiceException("Remote machine cannot be controlled")
        
        return app.servicecontrol.MachineController.forMachine(self, logger)


class Site(object):
    """ Represents the landscape configuration for the entire site (i.e., all machines) """

    def __init__(self, localMachine, allMachines):
        self.__localMachine = localMachine
        self.__allMachines = allMachines

    def getMachine(self, physicalName):
        if physicalName == "localhost":
            return self.getLocalMachine()

        for machine in self.__allMachines:
            if machine.getPhysicalName() == physicalName:
                return machine

        return None

    def getAllMachines(self):
        return self.__allMachines

    def getLocalMachine(self):
        return self.__localMachine

    def getMachinesByService(self, serviceName):
        return [machine for machine in self.__allMachines if machine.hasService(serviceName)]

    def resolveEndpointDependencies(self):
        for machine in self.getAllMachines():
            for service in machine.getServices():
                for reqdEndpoint in service.getRequiredEndpoints():
                    # Find a machine providing this service
                    providerMachines = self.getMachinesByService(reqdEndpoint.getService())

                    if len(providerMachines) == 0:
                        if not machine.isLocal():
                            # Our knowledge about remotely deployed services is limited - maybe
                            # the user omitted something in the local site-options.conf, or maybe
                            # we do not have the manifest for one of the services. As this dependency
                            # is not really relevant to our local services, ignore it.
                            continue

                        if reqdEndpoint.isRequiredForDeployment():
                            raise UnsatisfiedDependencyException(
                                "Service '%s' on '%s' needs '%s', but this is not deplyed anywhere" %
                                (service.getName(), machine.getPhysicalName(), reqdEndpoint.getService()))
                        else:
                            # Nevermind, the service can run without.
                            continue

                    elif len(providerMachines) > 1:
                        # Be solipsistic and assume that the local service is always used. Note that
                        # "local" changes depending on the viewpoint. 
                        # This becomes important in scenarios like registering an integration server.
                        if self.getLocalMachine() in providerMachines:
                            providerMachine = self.getLocalMachine()
                        else:
                            raise AmbiguousDependencyException(
                                "Service '%s' (needed by %s) is provided on multiple machines (%s) - do not know which one to use" %
                                (reqdEndpoint.getService(), service.getName(), ", ".join([m.getPhysicalName() for m in providerMachines])))
                    else:
                        providerMachine = providerMachines[0]

                    # Now that we know the machine, find the right endpoint.
                    providerService = providerMachine.getService(reqdEndpoint.getService())

                    if providerMachine == machine:
                        # Same machine - in this case, prefer an 'internal' endpoint.
                        scopePreference = ["internal", "public"]
                    else:
                        # Difference machines - prefer a 'public' endpoint because an 'internal' one
                        # may either not be reachable (firewall, binding) or may not be appropriately secured.
                        scopePreference = ["public", "internal"]

                    for scope in scopePreference:
                        providerEndpoint = providerService.getProvidedEndpoint(reqdEndpoint.getId(), scope)
                        if providerEndpoint != None:
                            break

                    if providerEndpoint == None:
                        raise MissingEndpointException(
                            "Service '%s' on '%s' needs '%s' which was found on %s, but it does not expose %s" %
                            (service.getName(), machine.getPhysicalName(), reqdEndpoint.getService(),
                             providerMachine, reqdEndpoint.getId()))

                    # We have a match
                    reqdEndpoint.snapProvider(providerService, providerEndpoint)
                    providerEndpoint.snapClient(service)
             
    def getMachineStartupOrder(self):
        # Build a dict `service : set([dependencies...])`...
        dependencyGraph = dict((machine, machine.getMachineStartupDependencies()) 
                               for machine in self.getAllMachines())

        # ...and sort it topologically. Finally, throw out anything that is not a daemon.
        import ext.toposort
        try:
            return ext.toposort.toposort_flatten(dependencyGraph)
        except ValueError:
            raise CyclicDependencyInMachineStartupOrder(
                "Cyclic dependency found in machine startup order - consider using a different landscape")
    
    def getController(self, logger):
        return app.servicecontrol.SiteController.forSite(self, logger)

    @staticmethod
    def fromSiteOrRuntimeOptions(siteOrRunimeOptions, serviceManifestCollection):
        return LegacyHostTokenParser(siteOrRunimeOptions, serviceManifestCollection).parse()

class LegacyHostTokenParser(object):
    appsToServicesMapping = {
            "app": "ctfcore",
            "database": "ctfcore-database",
            "datamart": "ctfcore-datamart",
            "cvs": "cvs",
            "subversion": "subversion",
            "etl": "etl",
            "codesearch": "bdcs",
            "gerrit": "gerrit",
            "binary": "binary",
            "indexer": "search"
        }

    def __init__(self, siteOrRunimeOptions, serviceManifestCollection):
        self.__serviceManifests = serviceManifestCollection
        self.__hostTokens = {}

        # Ensure that any {__OTHER_PROP__} referenes are resolved.
        self.__options = siteOrRunimeOptions.resolveReferences()
 
    def __getDatabaseType(self):
        type = self.__options.get("DATABASE_TYPE", None)
        if not type:
            return "postgres"
        elif type == "postgresql":
            return "postgres"
        elif type == "oracle":
            return "oracle"
        else:
            raise ConfigurationException("Unrecognized database type '%s'" % type)

    def __isDatamartAndDatabaseCohostedOnThisMachine(self, appsList):
        defaultPort = app.defaults.POSTGRES_DEFAULT_PORT

        return "database" in appsList \
            and "datamart" in appsList \
            and self.__getDatabaseType() == "postgres" \
            and self.__options.get("DATABASE_PORT", defaultPort) == self.__options.get("REPORTS_DATABASE_PORT", defaultPort)

    def __getServicesFromHostToken(self, appsList):
        services = []

        for app in appsList:
            if LegacyHostTokenParser.appsToServicesMapping.has_key(app):
                # Translate to service
                services += [LegacyHostTokenParser.appsToServicesMapping[app]]
            else:
                # Not an app, must be a service
                services += [app]

        if "database" in appsList:
            # "database" is just an alias - replace by concrete service, this
            # will in turn trigger the database alias to be added again later
            services.remove("ctfcore-database")
            services += ["ctfcore-database-" + self.__getDatabaseType()]
            
        if "datamart" in appsList:
            # "datamart" is just an alias - replace by concrete service, this
            # will in turn trigger the database alias to be added again later
            services.remove("ctfcore-datamart")
            
            if self.__isDatamartAndDatabaseCohostedOnThisMachine(appsList):                
                # Use cohosted service
                services += ["ctfcore-datamart-postgres-cohosted"]
            else:
                # Use dedicated service
                services += ["ctfcore-datamart-" + self.__getDatabaseType()]
        
        if "gerrit-database" in appsList:
            # "gerrit-database" is just an alias - replace by concrete service, this
            # will in turn trigger the database alias to be added again later
            services.remove("gerrit-database")

            # Gerrit does not support Oracle - so we have to hardcode Postgres here.
            services += ["gerrit-database-postgres"]

        if "reviewboard-database" in appsList:
            # reviewboard-database" is just an alias - replace by concrete service, this^M
            # will in turn trigger the database alias to be added again later
            services.remove("reviewboard-database")

            # Reviewboard doesnot support oracle - we have to hardcode postgres
            services += ["reviewboard-database-postgres"]


        allApps = " ".join(self.__hostTokens.values())
        if "app" in appsList:
            # We want the database administration tools to be installed where 
            # ctfcore is deployed. Technically, it does not matter where the
            # database administration tools are deployed, but this has been the
            # convention.
            services += ["ctfcore-database-admin-" + self.__getDatabaseType()]

            if "datamart" in allApps:
                # The datamart is installed somewhere (does not matter on which host,
                # so add the datamart administration tools as well.
                services += ["ctfcore-datamart-admin-" + self.__getDatabaseType()]

            if not "indexer" in allApps and not "search" in allApps:
                # If indexer/search is not explicitly listed anywhere, then
                # install it on the app box.
                services += ["search"]

        if not "service-monitor" in allApps:
            if self.__options.has_key("ENABLE_SERVICE_MONITORING") and self.__options["ENABLE_SERVICE_MONITORING"] == "true":
                services += ["service-monitor"]

        if "gerrit" in appsList and not "gerrit-database" in allApps:
            # The 'gerrit' token used to imply a local database - so if we do not have 
            # 'gerrit-database' listed in any HOST token, assume the database is supposed
            # to be installed locally.
            services += ["gerrit-database-postgres"]

   
        if "database" in appsList and "reviewboard" in allApps and not "reviewboard-database" in allApps:
            # If review-board database is not deployed anywhere then deploy it on 
            # the machine where ctfcore-database is running
            services += ["reviewboard-database-postgres"]


        return services

    def __loadDaemonStartupInfo(self, manifest):
        if not manifest.isDaemon():
            return None
        elif manifest.getDaemonInfo().isExternal():
            return DaemonStartupInfo(True, None, None, None)
        else:
            return DaemonStartupInfo(
                manifest.getDaemonInfo().isExternal(),
                manifest.getDaemonInfo().getUser().resolveValue(self.__options),
                manifest.getDaemonInfo().getGroup().resolveValue(self.__options),
                manifest.getDaemonInfo().getStartupTimeout())

    def __loadService(self, machine, serviceName):
        manifest = self.__serviceManifests.get(serviceName)
        
        reqdEndpoints = [RequiredEndpoint(
                            endpInfo.getId(), 
                            endpInfo.getService(),
                            endpInfo.isRequiredForDeployment(),
                            endpInfo.isRequiredForStartup()) 
                         for endpInfo in manifest.getRequiredEndpoints()]

        provEndpoints = [ProvidedEndpoint(
                            endpInfo.getId(), 
                            endpInfo.getScope(), 
                            machine, 
                            int(endpInfo.getPort().resolveValue(self.__options))
                                if endpInfo.getPort() else None,
                            endpInfo.getVia())
                         for endpInfo in manifest.getProvidedEndpoints()]

        return Service(
            serviceName, 
            self.__loadDaemonStartupInfo(manifest),
            machine, 
            provEndpoints, 
            reqdEndpoints,
            manifest.getIntegrityConfFiles(),
            self.__serviceManifests.isAlias(serviceName))
            
    def __purgeUnrecognizedServices(self, serviceNames):
        return [s for s in serviceNames if self.__serviceManifests.get(s) != None]

    def parse(self):
        self.__hostTokens = dict((key, value) for key, value in self.__options.iteritems() if key.rfind("HOST_") == 0)

        machines = []
        localMachine = None

        # Load config for each machine.
        for key, value in self.__hostTokens.iteritems():
            apps = set(value.split())

            # Determine physical host name
            physicalName = key.split("_")[1]
            isLocal = physicalName == "localhost" or physicalName == Machine.getLocalHostName()

            # Determine installed services from HOST_ tokens.
            serviceNames = self.__getServicesFromHostToken(apps)

            # Now that we know what gets installed, resolve aliases. On remote hosts, ignore
            # any aliases that we cannot resolve because we might not have all the manifests
            # locally.
            serviceNames = self.__serviceManifests.resolveAliasesForInstalledServices(serviceNames, not isLocal)

            if not isLocal:
                # Purge any services which we do not have manifests for.
                serviceNames = self.__purgeUnrecognizedServices(serviceNames)

            # Resolve deployment dependencies
            serviceNames = self.__serviceManifests.getTransitiveDeploymentDependencySet(serviceNames, ignoreUnrecognizedServices=not isLocal)
            
            # Determine public host name
            if isLocal and self.__options.has_key("DOMAIN_localhost"):
                publicName = self.__options["DOMAIN_localhost"]
            else:
                publicName = self.__options.get("DOMAIN_%s" % physicalName, physicalName)
            
            # Create machine and service objects
            machine = Machine(physicalName, publicName, isLocal)
            machine.setServices(
                [self.__loadService(machine, serviceName) for serviceName in serviceNames],
                self.__serviceManifests.getDeploymentOrder(serviceNames))

            # Now that all services are loaded, wire up the deployment dependencies (non-transitive)
            for service in machine.getServices():
                transitiveDependencyServiceNames = self.__serviceManifests.getTransitiveDeploymentDependencySet(
                    [service.getName()], 
                    False,
                    not isLocal)
                dependencyServiceNames = self.__serviceManifests.get(service.getName()).getDeploymentDependencies()
                containerServiceName = self.__serviceManifests.get(service.getName()).getContainer()
                if containerServiceName and containerServiceName not in dependencyServiceNames:
                    raise ConfigurationException(
                        "Service %s is hosted in container %s, but it lacks a corresponding deployment dependency" %
                        (service.getName(), containerServiceName))
                
                if not isLocal:
                    # Purge any services which we do not have manifests for.
                    dependencyServiceNames = self.__purgeUnrecognizedServices(dependencyServiceNames)
                    transitiveDependencyServiceNames = self.__purgeUnrecognizedServices(transitiveDependencyServiceNames)
                    containerServiceName = containerServiceName if self.__serviceManifests.get(containerServiceName) else None

                if dependencyServiceNames:
                    service.setDeploymentDependencies(
                        [machine.getService(name) for name in dependencyServiceNames],
                        [machine.getService(name) for name in transitiveDependencyServiceNames],
                        machine.getService(containerServiceName) if containerServiceName else None)

            machines.append(machine)
            if isLocal:
                localMachine = machine

        if localMachine == None:
            raise ConfigurationException("Missing HOST token for local machine")

        return Site(localMachine, machines)

    @staticmethod
    def setAdditionalTokens(siteOrRunimeOptions):

        # Quirk: If SSL is on, the HTTPD_PORT default value changes.
        # Todo: Turn into a runtime-option-override
        if siteOrRunimeOptions.get("SSL", "off") == "on" and not siteOrRunimeOptions.has_key("HTTPD_PORT"):
            siteOrRunimeOptions = siteOrRunimeOptions.overlayBy(
                app.options.RuntimeOptions({"HTTPD_PORT": "443"}))
        
        return siteOrRunimeOptions

class Landscape(object):
    """ Singleton object describing the current landscape """

    @staticmethod
    def isInitialized():
        try:
            Landscape.getSite()
            return True
        except:
            return False

    @staticmethod
    def initializeFromSiteOptionsPath(siteOptionsPath):
        # Note: site-options.conf tokens may reference each other using {__OTHERTOKEN__} syntax,
        # so these references need to be resolved. SITE_DIR used to be an implicit token (derived
        # from the -d command line switch, so this needs to be made available for resolution to 
        # work in all cases.
        siteOptions = app.options.SiteOptions \
            .readFromFilePath(siteOptionsPath, True) \
            .set("SITE_DIR", app.defaults.DEFAULT_SITE_DIR) \
            .resolveReferences()

        # Validate that the site-options.conf does not contain any obvious errors.
        tokenDef = app.tokens.defines.TokenDef()
        siteOptions = siteOptions.validate(tokenDef, app.util.getPlatform())

        # Initialize statics.
        Landscape.__siteOptionsPath = siteOptionsPath
        Landscape.__siteOptions = LegacyHostTokenParser.setAdditionalTokens(siteOptions)
        Landscape.__site = app.landscape.Site.fromSiteOrRuntimeOptions(
            Landscape.__siteOptions,
            app.servicemanifest.ServiceManifestCollection().loadFromDefaultLocation())
        Landscape.__site.resolveEndpointDependencies()

    @staticmethod
    def getSite():
        return Landscape.__site

    @staticmethod
    def getSiteOptions():
        return Landscape.__siteOptions

    @staticmethod
    def getSiteOptionsPath():
        return Landscape.__siteOptionsPath


class ConfigurationException(Exception):
    pass

class UnresolvedEndpointDependenciesException(Exception):
    pass

class UnsatisfiedDependencyException(ConfigurationException):
    pass

class AmbiguousDependencyException(ConfigurationException):
    pass

class MissingEndpointException(ConfigurationException):
    pass

class CyclicDependencyInMachineStartupOrder(ConfigurationException):
    pass

class NetworkException(ConfigurationException):
    pass

class ServiceException(Exception):
    pass
