﻿import os
import json
import string 
import app.defaults

MANIFEST_FILENAME = "manifest.json"

# Available to the outside world
PROVIDED_ENDPOINT_SCOPE_PUBLIC = "public"

# Available to other boxes via some local network
PROVIDED_ENDPOINT_SCOPE_INTERNAL = "internal"

REQUIRED_ENDPOINT_SCOPE_DEFAULT = "default"
REQUIRED_ENDPOINT_SCOPE_VITAL = "vital"
REQUIRED_ENDPOINT_SCOPE_OPTIONAL = "optional"

DEFAULT_STARTUP_TIMEOUT = 10

class Via(object):
    def __init__(self, service, id):
        self.__service = service
        self.__id = id
        
    def getId(self):
        return self.__id
    
    def getService(self):
        return self.__service

class ProvidedEndpointInfo(object):
    def __init__(self, id, scope, port, via):
        self.__id = id
        self.__scope = scope
        self.__port = port
        self.__via = via

    def getId(self):
        return self.__id
    
    def getScope(self):
        return self.__scope
    
    def getPort(self):
        return self.__port

    def getVia(self):
        return self.__via

class RequiredEndpointInfo(object):
    def __init__(self, id, service, scope):
        self.__id = id
        self.__service = service
        self.__scope = scope

    def getId(self):
        return self.__id
    
    def getService(self):
        return self.__service

    def isRequiredForStartup(self):
        return self.__scope == REQUIRED_ENDPOINT_SCOPE_VITAL

    def isRequiredForDeployment(self):
        return self.__scope != REQUIRED_ENDPOINT_SCOPE_OPTIONAL

class Option(object):
    def __init__(self, default, option):
        self.__default = default
        self.__option = option

    def getDefault(self):
        return self.__default

    def getOption(self):
        return self.__option

    def resolveValue(self, siteOrRuntimeOptions):
        if self.__option and siteOrRuntimeOptions.has_key(self.__option):
            return siteOrRuntimeOptions[self.__option]
        else:
            return self.__default

class DaemonInfo(object):
    def __init__(self, isExternal, user, group, startupTimeout):
        self.__isExternal = isExternal
        self.__user = user
        self.__group = group
        self.__startupTimeout = startupTimeout

    def isExternal(self):
        return self.__isExternal

    def getUser(self):
        return self.__user

    def getGroup(self):
        return self.__group

    def getStartupTimeout(self):
        return self.__startupTimeout
    
class ServiceManifest(object):
    """ Accessor object for manifest.json """

    def __init__(self, json):
        self.__manifestInfo = json

    @staticmethod 
    def readFromString(jsonString):
        return ServiceManifest(json.JSONDecoder().decode(jsonString))
    
    @staticmethod
    def readFromFilePath(filePath):
        with open(filePath) as file:
            return ServiceManifest(json.JSONDecoder().decode(file.read()))
        
    def getAlias(self):
        return self.__manifestInfo.get("alias", None)
    
    def getName(self):
        return self.__manifestInfo.get("name", None)

    def isDaemon(self):
        return self.__manifestInfo.has_key("daemon")

    def getContainer(self):
        if self.__manifestInfo.has_key("deployment"):
            return self.__manifestInfo["deployment"].get("container", None)
        else:
            return None

    def getDaemonInfo(self):
        if not self.isDaemon():
            return None

        return DaemonInfo(
            self.__manifestInfo["daemon"].get("external", False),
            Option(self.__manifestInfo["daemon"]["user"].get("default"),
                    self.__manifestInfo["daemon"]["user"].get("option"))
                if self.__manifestInfo["daemon"].has_key("user") else None,
            Option(self.__manifestInfo["daemon"]["group"].get("default"),
                    self.__manifestInfo["daemon"]["group"].get("option"))
                if self.__manifestInfo["daemon"].has_key("group") else None,
            self.__manifestInfo["daemon"].get("startupTimeout", DEFAULT_STARTUP_TIMEOUT))


    def getDeploymentDependencies(self):
        if self.__manifestInfo.has_key("deployment") and \
                self.__manifestInfo["deployment"].has_key("dependencies"):
            return self.__manifestInfo["deployment"]["dependencies"]
        else:
            return []

    def getRequiredEndpoints(self):
        dependencies = []
        
        if self.__manifestInfo.has_key("endpoints") and self.__manifestInfo["endpoints"].has_key("required"):
            for dependency in self.__manifestInfo["endpoints"]["required"]:
                dependencies.append(RequiredEndpointInfo(
                    dependency["id"], 
                    dependency["service"],
                    dependency.get("scope", REQUIRED_ENDPOINT_SCOPE_DEFAULT)))

        return dependencies

    def getProvidedEndpoints(self):
        endpoints = []

        if self.__manifestInfo.has_key("endpoints") and self.__manifestInfo["endpoints"].has_key("provided"):
            for endpoint in self.__manifestInfo["endpoints"]["provided"]:
                endpointInfo = None
                if endpoint.has_key("via"):
                    if endpoint.has_key("port"):
                        raise ServiceManifestException("Via cannot be combined with a default port")

                    endpointInfo = ProvidedEndpointInfo(
                        endpoint["id"], 
                        endpoint["scope"], 
                        None,
                        Via(endpoint["via"]["service"], endpoint["via"]["id"]))
                else:
                    endpointInfo = ProvidedEndpointInfo(
                        endpoint["id"], 
                        endpoint["scope"], 
                        Option(
                            endpoint["port"]["default"], 
                            endpoint["port"].get("option", None)),
                        None)

                endpoints.append(endpointInfo)

        return endpoints

    def getIntegrityConfFiles(self):
        if self.__manifestInfo.has_key("integrity"):
            return self.__manifestInfo["integrity"]


class ServiceManifestCollection(object):

    def __init__(self):
        self.__manifests = {}

    def add(self, serviceName, manifest):
        self.__manifests[serviceName] = manifest

    def resolveAliasesForInstalledServices(self, installedServiceNames, ignoreUnrecognizedServices=False):
        aliasesResolved = []

        for serviceName in installedServiceNames:
            manifest = self.get(serviceName)

            if not manifest:
                if ignoreUnrecognizedServices:
                    # Nervermind.
                    pass
                else:
                    raise ServiceManifestException("Unknown service '%s'" % serviceName)
            elif manifest.getAlias():
                self.__manifests[manifest.getAlias()] = manifest
                aliasesResolved.append(manifest.getAlias())


        return installedServiceNames + aliasesResolved

    def isAlias(self, serviceName):
        manifest = self.get(serviceName)
        if manifest:
            return manifest.getAlias() == serviceName
        else:
            return False

    def get(self, serviceName):
        return self.__manifests.get(serviceName, None)

    def getServiceNames(self):
        return self.__manifests.keys()

    def getTransitiveDeploymentDependencySet(self, serviceNames, includeRoots=True, ignoreUnrecognizedServices=False):
        """ Returns the transitive set of dependencies, including the root services """
        
        def walk(service, servicesVisited, serviceAccumulator, manifestDict):
            if not service in servicesVisited:
                # Make sure we do not visit any service twice
                servicesVisited.add(service)
                
                dependencies = manifestDict[service].getDeploymentDependencies()
                if ignoreUnrecognizedServices:
                    # Purge unknown dependencies.
                    dependencies = [d for d in dependencies if manifestDict.has_key(d)]
                else:
                    # Bail out if there is an unknown dependency.
                    for service in [d for d in dependencies if not manifestDict.has_key(d)]:
                        raise ServiceManifestException("Manifest for service '%s' not available for dependency resolution" % service)

                serviceAccumulator += dependencies
                
                #print "%s depends on %s -> acc %s [after visting %s]" %(service, ",".join(dependencies), ",".join(serviceAccumulator), ",".join(servicesVisited))

                for dependency in dependencies:
                    walk(dependency, servicesVisited, serviceAccumulator, manifestDict)

        if includeRoots:
            # Start with the list of services
            accumulator = serviceNames
        else:
            accumulator = []

        # Now walk dependency tree of each service to gather transitive set of dependencies
        visited = set()
        for serviceName in serviceNames:
            walk(serviceName, visited, accumulator, self.__manifests)

        # Remove duplicates
        return set(accumulator)

    def getDeploymentOrder(self, serviceNames):
        # Build a dict `name : set([dependencies...])` and sort it topologically
        dependencyGraph = dict((serviceName, set(self.get(serviceName).getDeploymentDependencies())) for serviceName in self.getServiceNames())
        
        
        import ext.toposort
        return ext.toposort.toposort_flatten(dependencyGraph)

    @staticmethod
    def loadFromDefaultLocation():
        serviceNames = [service for service 
                        in os.listdir(app.defaults.DEFAULT_SERVICE_DIR)
                        if os.path.isdir(os.path.join(app.defaults.DEFAULT_SERVICE_DIR, service))
                        and os.path.isfile(os.path.join(app.defaults.DEFAULT_SERVICE_DIR, service, MANIFEST_FILENAME))]

        collection = ServiceManifestCollection()

        for serviceName in serviceNames:
            manifestPath = os.path.join(app.defaults.DEFAULT_SERVICE_DIR, serviceName, MANIFEST_FILENAME)
            collection.add(serviceName, ServiceManifest.readFromFilePath(manifestPath))

        return collection

class ServiceManifestException(Exception):
    pass
