import os
import signal
import time
import app.servicecontrol

START_TIMEOUT = 20

class ServiceController(app.servicecontrol.ServiceController):
    def __init__(self, service, logger):
        super(ServiceController, self).__init__(service, logger)
        
        self.__lastStartupTime = 0
        
    def getPidFile(self):
        for pidLocation in ["/var/run/httpd.pid", "/var/run/httpd/httpd.pid"]:
            if os.path.isfile(pidLocation):
                return pidLocation
                
        return None

    def getKillSignal(self):
        # Apache leave some child processes, so send TERM signal to the parent causes it to 
        # attempt to kill off all its children immediately.
        return signal.SIGTERM
        
    def canBootstrap(self):
        # Bootstrapping Apache is a no-op, so allow it always.
        return True
       
    def isStarting(self):
        # Apache does not immediately publish its PID file - but that does not mean
        # it is in trouble. So we need to give it some extra time before making any status
        # decisions.
        return not self.getPid() and (time.time() - self.__lastStartupTime) < START_TIMEOUT

    def __runInit(self, verb):
        config = app.util.getRuntimeConfiguration()

        logFile = os.path.join(config['HTTPD_LOG_DIR'], 'service.log')
        initScript = config["HTTPD_SERVICE_CMD"]

        return os.system("%s %s < /dev/null >> %s 2>&1 &" % (initScript, verb, logFile)) == 0
        
    def startDaemonAsync(self):
        if self.__runInit("start"):
            self.__lastStartupTime = time.time()
            return True
        else:
            return False
        
    def stopDaemonAsync(self):
        return self.__runInit("stop")
        
    def createPostDeployAction(self):
        # Start the service right after deployment
        return RestartApacheAction(self.getService(), self.logger)
        
class RestartApacheAction(app.servicecontrol.ControllerAction):
    def __init__(self, service, logger):
        self.__service = service
        return super(RestartApacheAction, self).__init__(logger)
    
    def getInvolvedServices(self):
        return [self.__service]
    
    def confirm(self, isRedeployment=False):
        return True

    def execute(self):
        import app.actions.stop
        import app.actions.start
        
        return app.actions.stop.StopServiceAction([self.__service], self.logger).execute() and \
               app.actions.start.StartServiceAction([self.__service], self.logger).execute()
