#!/bin/bash

# Script that does bootstrap of Codesearch Application
# There is not direct method do Codesearch bootstrapping as per V2.1.1
# The following is the  procedure to do bootstrapping codesearch
#   1. Backup the License file. Do Codesearch Uninstall
#   2. Remove Codesearch Postgresql, Scan and Install Directory
#   3. Do a fresh Codesearch installation with Property file from 
#      $RUNTIME_DIR/conf/codesearch.properties
#   4. Apply the license file back
#   5. Restart the Codesearch Application

# Source the tokens
eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"
# Setting Internal Field Separator to preserve white spaces in variables
IFS='%'

# Function to Initialise the Variables used at various functions in this script
initialise_vars() {
  codesearchRpm="CodeSearch-installer"
  bdcsVersion=`rpm -qa --queryformat "%{version}" $codesearchRpm 2> /dev/null`
  if [ -z $bdcsVersion ]; then
     log_message "Error: $codesearchRpm package is not installed!!"
     log_message "       Please install $codesearchRpm rpm first."
     log_message "Hint:  Run \"yum install $codesearchRpm\""
     exit 1
  fi
  bdcsUnInstallScript="$BDCS_INSTALL_PATH/CodeSight/uninstall/uninstall_CodeSight"
  bdcsInstallScript="$BDCS_INSTALLER_PATH/$bdcsVersion/install.sh"
  bdcsInstallerPropertyFile="${RUNTIME_DIR}/conf/codesearch.properties"
  bdcsAppDir=$BDCS_INSTALL_PATH/CodeSight
  bdcsDataDir=$BDCS_PGSQL_HOME_DIR_ROOT
  bdcsScanDir=$BDCS_SCAN_SOURCE_DIR_ROOT
  bootstrapLogFile=$BDCS_LOG_DIR/bootstrap.log
  bdcsLicenseFile="${BDCS_INSTALL_PATH}/CodeSight/config/CodeSight_v1.xml"
  bdcsLicenseFileBak="${SITE_DIR}/etc/CodeSight_v1.xml"
  bdcsLddFile="/etc/ld.so.conf.d/bds-codesight-postgresql.conf"
  ldconfigCmd=`which --skip-alias ldconfig`
  cpCommand=`which --skip-alias cp`
  rmCommand=`which --skip-alias rm`
  cntrlCommand="${RUNTIME_DIR}/scripts/codesearch.sh"

  log_message
  log_message "Auto Detected CodeSearch Settings"
  log_message "================================="
  log_message "   Installer Path :     $BDCS_INSTALLER_PATH"
  log_message "   Installer Version :  $bdcsVersion"
  log_message "   Installed Location : $bdcsAppDir"
  log_message "   Data Directory :     $bdcsDataDir"
  log_message "   Scan Directory :     $bdcsScanDir"
  log_message

  unset DISPLAY
}

# Function to Delete CodeSearch Files
delete_codesearch_files() {
  if [ ! -z $bdcsDataDir ]; then
     log_message -en "Removing Codesearch Data Directories ... "
     $rmCommand -rf $bdcsDataDir 2> /dev/null
     log_message "Done."
  fi

  if [ ! -z $bdcsScanDir ]; then
     log_message -en "Removing Codesearch Scan Directories ... "
     $rmCommand -rf $bdcsScanDir 2> /dev/null
     log_message "Done."
  fi

  if [ ! -z $bdcsAppDir ]; then
     log_message -en "Removing Codesearch App Directories ... "
     $rmCommand -rf $bdcsAppDir 2> /dev/null
     log_message "Done."
  fi
}

# Function to Backup CodeSearch License File
backup_license() {
  log_message -en "Backup CodeSearch License File ... "
  $cpCommand $bdcsLicenseFile $bdcsLicenseFileBak -axf 2> /dev/null
  log_message "Done."

  return 0
}

# Function to Uninstall Codesearch App
uninstall_codesearch() {
  log_message -en "Uninstalling Codesearch Application. Please wait ... "
  $bdcsUnInstallScript 2> /dev/null
  log_message "Done"

  return 0
}

# Function to Install Codesearch App
install_codesearch() {
  log_message -en "Installing Codesearch Application. Please wait ... "
  $bdcsInstallScript -i silent -Dblackduck.installer.properties=$bdcsInstallerPropertyFile > /dev/null 2>&1

  if [ $? != 0 ]; then
     log_message "Failed!"
     log_message "Error occurred while installing CodeSearch."
     log_message "Please see the log file /tmp/BDS_IACA_CodeSight_server.log for details"
     exit 1
  else
     log_message "Done"
  fi
  return 0
}

# Function to check if Codesearch App is running in this box
is_codesearch_enabled() {
  codeSearchEnabled=`grep HOST_${NODE_HOSTNAME} $RUNTIME_DIR/conf/runtime-options.conf 2> /dev/null | \
                             cut -d'=' -f2 | grep 'codesearch' | wc -l`
  if [ ${codeSearchEnabled} -ne "1" ]; then
     return 1
  else
     return 0
  fi
}

# Function to check if the user running this script is a root user.
verify_run_permission() {
   if [ `id -u` -ne 0 ]; then
      echo "Permission Denied to run this program."
      echo
      echo "NOTE: You must run this script as root!"
      exit 1
   fi
   return 0
}

# Function to restore Codesearch License file
restore_license() {
   log_message -en "Restoring CodeSearch License File ... "
   $cpCommand $bdcsLicenseFileBak $bdcsLicenseFile -axf 2> /dev/null
   ret=$?
   if [ $ret -ne 0 ]; then
      log_message "Failed!"
      log_message "Use license-utils.sh to install Codesearch License manually."
      return 1
   else
      log_message "Done."
      return 0
   fi
}

# Function to Restart Codesearch application
restart_codesearch() {
   log_message -en "Restarting Codesearch Application ... "
   $cntrlCommand stop > /dev/null 2>&1
   sleep 2
   $cntrlCommand start > /dev/null 2>&1
   log_message "Done."
}


# Function to check if Codesearch App is running in this box
verify_codesearch_enabled() {
  codeSearchEnabled=`grep HOST_${NODE_HOSTNAME} $RUNTIME_DIR/conf/runtime-options.conf 2> /dev/null | \
                             cut -d'=' -f2 | grep 'codesearch' | wc -l`
  if [ ${codeSearchEnabled} -ne "1" ]; then
     log_message "Error: Codesearch is not enabled in this box"
     exit 1
  else
     return 0
  fi
}

# Function to Prompt and Stop Codesearch application
prompt_codesearch_bootstrap() {

   log_message "NOTE: Bootstrap Codesearch will remove all data like Database,"
   log_message "      Index Files, Scan Files and creates a fresh instance of"
   log_message "      Codesearch application"
   log_message

   if [ $promptUser = 0 ]; then
      return 1
   fi

   log_message -en "Shall I Proceed (Y/N) ? "
   read option

   case "${option}" in
     Y)
       return 1
       ;;
     N)
       log_message "CodeSearch Bootstrap Cancelled."
       exit 1
       ;;
     *)
       log_message "Invalid input!! CodeSearch Bootstrap Cancelled."
       exit 1
       ;;
   esac
}

# Function to display the usage help of this script
print_usage() {
   progName=`basename $0`
   echo "Usage: $progName [OPTION]"
   echo "BootStrap CodeSearch Application."
   echo
   echo "       --force             Force bootstrap without prompt"
   echo
}

log_message() {
   echo $1 $2 | tee -a $bootstrapLogFile
}

promptUser=1
if [ ! -z $1 ]; then
   case "$1" in
     --force)
         promptUser=0
         ;;
     *)
         log_message "Error: Invalid option provided!!"
         print_usage
         exit 1
         ;;
   esac
fi

# Check if the User has permission to run this script. 
# Only Root can run this script
verify_run_permission

# Run only if Codesearch application is enabled in this box.
verify_codesearch_enabled

# Initilise Variables and Environments
initialise_vars

# Prompt the user before proceeding with Codesearch Bootstrap process
prompt_codesearch_bootstrap

curTime=`date`
log_message "Started Codesearch Bootstrap at: $curTime"
# Take the backup of Codesearch License file if any..
backup_license

# Uninstall Codesearch Application
uninstall_codesearch

# Delete Codesearch Files (Database, Scan, App)
delete_codesearch_files

# Install Codesearch Application
install_codesearch

# Restore License File
restore_license

# Restart Codesearch Applicaton
restart_codesearch

log_message "Codesearch Application Bootstrapped Successfully!!!"
curTime=`date`
log_message "Completed Codesearch Bootstrap at: $curTime"

exit 0;
