#!/bin/bash

# Script that provides Codesearch License related utility functions like
# Install Codesearch License
# Display the Codesearch License Id
# Display the List of all Macids available in the system

# Source the tokens
eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"

# Initialise the Variables used at various functions in this script
initialise_vars() {
   appLicenseFile="${BDCS_INSTALL_PATH}/CodeSight/config/CodeSight_v1.xml"
   cpCommand=`which --skip-alias cp`
   dateCommand=`which --skip-alias date`
   orgFilePrefix=`$dateCommand +%d%m%Y%H%M%S`
   cntrlCommand="${RUNTIME_DIR}/scripts/codesearch.sh"
}

# Function to install Codesearch License file
install_license() {
   verify_run_permission
   verify_license_file $1
   verify_codesight_installation

   prompt_codesearch_stop

   $cpCommand $1 $appLicenseFile -axf -bS".${orgFilePrefix}" 2> /dev/null
   ret=$?
   if [ $ret -ne 0 ]; then
      echo "Error: Unable to copy License file!!"
      exit 1
   else
      echo "Codesearch License File installed at ${BDCS_INSTALL_PATH}."
   fi

   start_codesearch
   return 0
}

# Function to Start Codesearch application
start_codesearch() {
   $cntrlCommand start
}

# Function to Prompt and Stop Codesearch application
prompt_codesearch_stop() {

   echo "NOTE: The Codesearch server needs to be shutdown before installing"
   echo "      the Codesearch License file."
   echo -en "Shall I Proceed (Y/N) ? "
   read option

   case "${option}" in
     Y)
       $cntrlCommand stop
       ;;
     N)
       echo "Install License Operation Cancelled."
       exit 1
       ;;
     *)
       echo "Invalid input!! Operation Cancelled."
       exit 1
       ;;
   esac
}

# Function to check if Codesearch is installed.
verify_codesight_installation() {
   if [ ! -d "$BDCS_INSTALL_PATH" ]; then
      echo "Error: Unable to determine Codesight Path!!"
      exit 1
   fi

   if [ ! -f "$BDCS_INSTALL_PATH/CodeSight/id.txt" ]; then
      echo "Error: Codesight not installed!!"
      exit 1
   fi
}

# Function to verify and display the Codesearch License Id
verify_license_file() {
   licenseId=`head -n 5 $1 2> /dev/null | grep "codesight-" | tr -d '<> '`
   if [ ! -z ${licenseId} ]; then
      echo "Your Codesearch License Id is [${licenseId}]"
      return 0
   else
      echo "Error: The Input License file is invalid!!"
      exit 1
   fi
}

# Function to display all the mac ids of the ethernet interfaces configured 
# in the system.
show_macids() {
   output=`/sbin/ifconfig 2> /dev/null | grep HWaddr | awk '{ print $1 " => " $5; }'`
   if [ $? = 0 ]; then
      echo "List of Ethernet Interfaces and MAC Addresses"
      echo "============================================="
      echo $output
   fi
 
}

# Function to check if the user running this script is a root user.
verify_run_permission() {
   if [ `id -u` -ne 0 ]; then
      echo "Permission Denied to run this program."
      echo
      echo "NOTE: You must run this script as root!"
      exit 1
   fi
}

# Function to display the usage help of this script
print_usage() {
   progName=`basename $0`
   echo "Usage: $progName [OPTION]"
   echo "Install/Show Codesearch License."
   echo
   echo "       --install <license_file_path>        Installs Codesearch License"
   echo "       --show                               Displays Codesearch License"
   echo "       --listmacid                          Displays MAC Ids of all Ethernet Interface"
   echo
}

initialise_vars
case "$1" in
  --install)
      if [ -z $2 ]; then
        print_usage
        exit 1
      fi
      install_license $2
      echo "Success: CodeSight license is installed sucessfully!!" 
      exit 0
      ;;
  --show)
      verify_license_file $appLicenseFile
      exit 0
      ;;
  --listmacid)
      show_macids
      exit 0
      ;;
  --help)
      print_usage
      exit 0
      ;;
  *)
      echo "Error: Invalid option provided!!"
      print_usage
      exit 1
      ;;
  esac
