#!/bin/bash

# Source the tokens
eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"

# Set shell option to enable "=~" conditional operator that is used for match regular expressions.
shopt -s compat31
# Set shell option to enable case insensitve pattern matching
shopt -s nocasematch

export JAVA_HOME
USERNAME="scmviewer"
LOG_FILE=$BDCS_LOG_DIR/post_install.log
CACHE_BASE_PATH=$HOME/.subversion/auth/svn.simple/

# Function to print messages both on tty and logfile.
debug_Print() {
    echo $1 $2 | tee -a $LOG_FILE
}

# Funtion that provides SCMViewer user password
get_SCMPass() {
   SCM_PASS=`$JAVA_HOME/bin/java -cp $UTIL_JAR_PATH com.vasoftware.sf.system.utils.ScmIntegrationAccessKeyProcessor -decrypt $1`; ret=$?
   if [ $ret -ge 1 ]; then
      debug_Print "Unable to detect SCM Viewer Password."
      debug_Print "Reason: $SCM_PASS"
      return 1;
   else
      return 0;
   fi
}

# Function that generates the auth realm parameter for svn repository from the provided svn url
get_AuthRealm() {
     svnUrl=$1
     urlRe='^(http(s)?)://([^/?#]*)(.*)$'

     if [[ ${svnUrl} =~ ${urlRe} ]]; then
       protocol=`echo ${BASH_REMATCH[1]} | tr '[:upper:]' '[:lower:]'`
       domain=`echo ${BASH_REMATCH[3]} | tr '[:upper:]' '[:lower:]'`
       port="80"
       if [ ${protocol} == 'https' ]; then
          port="443"
       fi

       svnBaseUrl="${protocol}://${domain}:${port}"
       curl ${svnBaseUrl} -k > /dev/null 2>&1;ret=$?;
       if [ $ret -ne "0" ]; then
          debug_Print "Integration Service is not running in [$svnBaseUrl]!"
          return 1
       fi
     else
       debug_Print "Invalid subversion repository url provided [$svnUrl]!"
       return 1
     fi

     AUTHREALM="<${svnBaseUrl}> Authorization Realm"
     return 0
}

open_CacheFile() {
    echo -n > $CACHE_BASE_PATH/$CACHEFILE
}

write_CacheFile() {
    echo $* >> $CACHE_BASE_PATH/$CACHEFILE
}

close_CacheFile() {
    echo "END" >> $CACHE_BASE_PATH/$CACHEFILE
}

# Function that stores information in key/value pair format.
add_CacheFile() {
   key=$1
   value=$2

   keylen=`echo -n $key | wc -c`
   valuelen=`echo -n $value | wc -c`

   write_CacheFile "K $keylen"
   write_CacheFile "$key"
   write_CacheFile "V $valuelen"
   write_CacheFile "$value"
}

print_Usage() {
   echo
   echo "Usage: ./svn_cache.sh <REPO_BASE_URL>"
   echo "Where: REPO_BASE_URL   -   Repository Base URL of your SCM integration Server"
   echo "                           (eg. https://myintegration.dev.collab.net/svn/repos)"
   echo
   echo "Note: 1. This program will cache svn credentials of remote scm integration server"
   echo "         so that your codesearch application would be able to checkout repo data"
   echo "         from the remote scm integration server for indexing purposes."
   echo "      2. This program does not authenticate the svn credentials, but only creates"
   echo "         authentication cache file based on SCM_USER_ENCRYPTED_PASSWORD option set"
   echo "         your runtime options confiugration."
}

# Check if help need to displayed
if [ -z $1 ] || [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
   print_Usage
   exit 0
fi

debug_Print "Starting Codesearch SVN Credentials Cache process"
debug_Print "================================================="
debug_Print "`date`"

# Create if .subversion path doesnot exists by calling svn command
if [ ! -d $CACHE_BASE_PATH ]; then
   svn help > /dev/null 2>&1
fi

inputUrl=$1
debug_Print "-en" "Fetching SCM Authentication Realm for [$inputUrl] ... "
get_AuthRealm $inputUrl; ret=$?
if [ $ret -ne "0" ]; then
   debug_Print "ERROR: Please verify if you have provided valid repository url and check if the Integration Host is running!!!"
   exit 1
fi
debug_Print "Done"

debug_Print "-en" "Fetching SCMViewer credentials ... "
get_SCMPass $SCM_USER_ENCRYPTED_PASSWORD; ret=$?
if [ $ret -ne 0 ]; then
   debug_Print "ERROR: Error occurred. Exiting !!!"
   exit 1
fi
debug_Print "Done"

debug_Print "-en" "Generating SVN Cache file ... "
CACHEFILE=`echo -n $AUTHREALM | md5sum | awk '{ print $1; }'`
open_CacheFile
add_CacheFile "passtype" "simple"
add_CacheFile "password" $SCM_PASS
add_CacheFile "svn:realmstring" "$AUTHREALM"
add_CacheFile "username" $USERNAME
close_CacheFile
debug_Print "Done"

debug_Print "CodeSearch SVN Credentials Cache process is successful."
debug_Print "======================================================="
