from app.scripts import BaseMigrationScript

import os
import hashlib
import app.util
from app.db import fetchAll, getConnection

# DB query for selecting profile picture file ids and hashable values
# for all users.
DB_SELECT_HASHDATA_FOR_AVATARS = """\
SELECT stored_file.raw_file_id,
       sfuser.username,
       sfuser.email,
       sfuser.alternate_email_1,
       sfuser.alternate_email_2,
       sfuser.alternate_email_3
  FROM sfuser_view sfuser,
       stored_file
 WHERE sfuser.profile_file_id = stored_file.id"""

# Location of the file we'll install as the "nobody" user's avatar.
NOBODY_IMAGE_PATH = os.path.join(os.path.dirname(__file__),
                                 'data', 'no_profile.png')

class MigrationScript(BaseMigrationScript):
    # This script is needed for migrating to CTF 8.1. It can be dropped once
    # updates from CTF 8.0 (or older) are not supported any more.
    
    scriptDescription = "Generate missing avatar images"

    def migrate(self):
        self.appUser = self.cfg['APP_USER']
        self.appGroup = self.cfg['APP_GROUP']
        self.avatarStore = self.cfg['AVATAR_SERVICE_DISK_STORE']
        self.filestorageLocation = self.cfg['FILESTORAGE_LOCATION']

        for fileName in [self.avatarStore,
                         os.path.join(self.avatarStore, 'original')]:
            self._createAvatarStorageDir(fileName)
        self._createProfileImageLinks()
        self._setStorePermissions()

    def _createAvatarStorageDir(self, fileName):
        if os.path.islink(fileName):
            fileName = os.readlink(fileName)
        if not os.path.exists(fileName):
            try:
                app.util.runSystem("mkdir -p %s" % (fileName), self.log)
            except:
                exc = ''.join(getStackTraceAndException())
                raise MigrationException("Failed to create Avatar service "
                                         "storage location:\n%s" % (exc))

    def _createProfileImageLinks(self):
        # Loop over all user records that have profile images.
        for record in fetchAll(getConnection(),
                               DB_SELECT_HASHDATA_FOR_AVATARS):
            # First item is the profile picture raw_file_id.
            # Everything else is something we want to make a symlink
            # for (or is empty).
            profilePicFilename = record[0]
            hashables = record[1:]

            # Generate the fullpath to the profile picture.
            profilePicPath = os.path.join(self.filestorageLocation,
                                          profilePicFilename[:1],
                                          profilePicFilename[:2],
                                          profilePicFilename[:3],
                                          profilePicFilename[:4],
                                          profilePicFilename)

            # Loop over the hashables for this profile picture,
            # creating any missing symlinks at the appropriate avatar
            # storage location.
            for hashable in hashables:
                if not hashable:
                    continue
                self._createImageLink(hashable, profilePicPath)

        # We want to also install a special image for the 'nobody'
        # user.
        self._createImageLink('nobody', NOBODY_IMAGE_PATH, True)

    def _createImageLink(self, hashable, profilePicPath, doCopy=False):
        digest = hashlib.md5(hashable.strip().lower()).hexdigest()
        linkParentDir = os.path.join(self.avatarStore, 'original',
                                     digest[:2])
        linkPath = os.path.join(linkParentDir, digest)
        if os.path.exists(profilePicPath) and \
           not os.path.exists(linkPath):
            app.util.runSystem("mkdir -p %s" % (linkParentDir), self.log)
            if doCopy:
              app.util.runSystem("cp %s %s" % (profilePicPath, linkPath), self.log)
            else:
              app.util.runSystem("ln -s %s %s" % (profilePicPath, linkPath), self.log)
        
    def _setStorePermissions(self):
        app.util.runSystem("chown -R %s:%s %s" %
                       (self.appUser, self.appGroup, self.avatarStore), self.log)
