#!/usr/bin/env python2

"""\
Usage: snapshot.py [--extra=command]
Usage: snapshot.py [--help|-h]
Usage: snapshot.py [--verbose|-v]

Parameters:

 extra      Extra command whose output should be placed in the log file.
            Enclose commands with options within quotes eg: 
            --extra '/usr/sbin/lsof -n -P -b -i -U'
 help       Prints this message and exits.
 
 verbose    Output each action of the function of  snapshot.py  

Prints a 'snapshot' of current state of CollabNet TeamForge and
system its running into the following logs:
  log/runtime/snapshot.log        Primary snapshot.py output log
  log/apps/server.log            JBoss JVM thread dump
This script can be called by 'watchdog'-type scripts at the time problems
are observed. The generated report is to help diagnosis.
                                                                                                                      

"""
__copyright__ = "Copyright (c) 1999-2006 CollabNet, Inc. All rights reserved."
import sys
import os
import re
import getopt
import time
import urllib2
import signal
import time
import logging

import app.util
import app.logutil

import service.util
import service.constants

verbose = False
checkCodeSearch = False

NEWLINE = '\n'
MODE = 'a+'

PG_STAT_SQLS_TF = {
    'pg_stat_activity': "select * FROM pg_catalog.pg_stat_activity",
    'pg_stat_database': "select * from pg_catalog.pg_stat_database",
    'pg_locks': "select * from pg_catalog.pg_locks",
    'pg_locks_stats'  : "select bl.pid as blocked_pid, bl.mode as blocked_mode, a.usename as blocked_user, ka.query as blocking_statement, now() - ka.query_start as blocking_duration, kl.pid as blocking_pid, kl.mode as blocking_mode, ka.usename as blocking_user, a.query as blocked_statement, now() - a.query_start as blocked_duration from pg_catalog.pg_locks bl join pg_catalog.pg_stat_activity a on bl.pid = a.pid join pg_catalog.pg_locks kl join pg_catalog.pg_stat_activity ka on kl.pid = ka.pid on bl.transactionid = kl.transactionid and bl.pid != kl.pid where not bl.granted"
}

PG_STAT_SQLS_OTHERS = {
    'pg_stat_activity': "select * FROM pg_catalog.pg_stat_activity"
}

def doJVMThreadDump(cfg, log, label):
    """Do JVM thread dump if running.
    """
    pids = []
    failures = []
    numberOfTries = 3
    while (numberOfTries > 0):
        (failures, pids) = app.util.getJVMPIDs()
        if not failures:
            break
        log.warn("Got invalid process list elements.. trying again")
        numberOfTries = numberOfTries -1
    if failures:
        log.warn("Giving up..")
        log.warn("Invalid ps output lines and stack trace are:") 
    if failures and verbose:   
        print "Giving up .." 
        print "Invalid ps output lines and stack trace are:"  
        for val in failures:
            log.warn(val)
    if not pids:
        log.info("No JVM process found.")
    else:
        # Jboss Thread dump is going to go into the jboss log file.
        # Set it off by surrounding the thread dump w/ easy to find delimiters.

        startStr = 'START of %s JVM thread dump.' % label
        endStr = 'END of %s JVM thread dump.' % label
        newline = '\n'
        mode = 'a+'

        jbossFile = os.path.join(cfg.get('APPLICATION_LOG_DIR'), 'jboss_console.log')
        integrationFile = os.path.join(cfg.get('INTEGRATION_LOG_DIR'), 'catalina.out')
	etlFile = os.path.join(cfg.get('ETL_LOG_DIR'), 'catalina.out')
        jamesFile = os.path.join(cfg.get('JAMES_LOG_DIR'), 'phoenix.console')
        if (checkCodeSearch):
           codesearchFile = os.path.join(cfg.get('BDCS_INSTALL_PATH'), 'CodeSight', 'tomcat', 'logs', 'catalina.out')
        appendToServiceLog(log, 'jboss', jbossFile, startStr, False)
        appendToServiceLog(log,'integration', integrationFile,startStr,True)
	appendToServiceLog(log,'tomcat_etl', etlFile,startStr,True)
        appendToServiceLog(log,'phoenix', jamesFile,startStr,True)
        if checkCodeSearch:
           appendToServiceLog(log,'tomcatcs', codesearchFile,startStr,True)
        cmd = 'kill -QUIT %s' % ' '.join(pids)
        errCode = os.system(cmd)
        if errCode:
            log.info('Got error code %d running %s' % (errCode, cmd))
        # Introduce a sleep of 2s to have the output in a consistent manner
        # despite CPU contention (it will take the JVMs some time to output
        # thread dumps; 2s won't always be enough, but should be sufficient
        # at least some of the time).  PCN22764
        time.sleep(2)

        appendToServiceLog(log,'jboss', jbossFile,endStr,True)
        appendToServiceLog(log,'integration', integrationFile,endStr,True)
	appendToServiceLog(log,'tomcat_etl', etlFile,endStr,True)
        appendToServiceLog(log,'phoenix', jamesFile,endStr,True)
        if checkCodeSearch:
           appendToServiceLog(log,'tomcatcs', codesearchFile,endStr,True)
        log.info("Find thread dump between lines demarked by '%s'." % label)

def appendToServiceLog(log, serviceName, logFile, msg, endOfThreadDump):
    """Appends the given message to the service's log file.
    
    """
    servicefh = open(logFile, MODE)
    servicefh.write(msg)
    servicefh.write(NEWLINE)
    servicefh.flush()
    servicefh.close()
    if endOfThreadDump:
        log.info("%s JVM threads dumped to %s." % (serviceName, logFile))

def doSourcecast(cfg, log, logFile):
    """Add state of collabnet to snapshot.
    
    Currently nothing to ask of collabnet.
    """
    pass

def readTimeoutHandler(signum, frame):
    """Called when we timeout reading from an url."""
    raise IOError, "Timed out the read."

def doApache(cfg, log):
    """Add state of apache to snapshot."""
    apachePort = 80
    if cfg.has_key('APACHE_PORT'):
        apachePort = cfg.get('APACHE_PORT')
    url = 'http://127.0.0.1:%s/server-status' % apachePort
    log.debug("Getting %s." % url)
    try:
        c = urllib2.urlopen(url)
        # Wrap the read w/ a timeout in case apache can't satisfy request:
        # e.g. its at MaxClients.
        signal.signal(signal.SIGALRM, readTimeoutHandler)
        signal.alarm(5)
        log.info(c.read()) 
        signal.alarm(0)
    except Exception, e:
        # Don't let any exception stop snapshotting completing.
        log.exception("Exception getting %s: %s" % (url, e))

def doDatabaseThreadDump(cfg, log):
    """Dump database threads"""

    pgsqlDatabaseHost = service.util.isServiceEnabledLocally(service.constants.SERVICE_PGSQL)
    appHost = service.util.isApplicationEnabledLocally(service.constants.APPLICATION_APP)

    for sqlKey in PG_STAT_SQLS_TF:
        if pgsqlDatabaseHost:
            cmdStr = "su postgres -c \"psql -c '%s'\"" % PG_STAT_SQLS_TF[sqlKey]
        elif appHost:
            #TODO: Fix artf148801
            psql_wrapper_path = cfg['RUNTIME_SCRIPTS']
            psql_wrapper = os.path.join(psql_wrapper_path, 'psql-wrapper')
            cmdStr = "echo \""+ PG_STAT_SQLS_TF[sqlKey] + "\" | " +  psql_wrapper
    
        if (pgsqlDatabaseHost or appHost):
            log.info("Start of database thread dump of [%s]."%(sqlKey))
            try:
                log.info(app.util.executeCommand(cmdStr))
            except:
                log.exception("Command failed: %s" % cmdStr)

def doReportingDatabaseThreadDump(cfg, log):
    """Dump reporting database threads"""

    datamartEnabled = service.util.isServiceEnabled(service.constants.SERVICE_REPORTS_PGSQL,
                service.util.getApplicationHost(service.constants.APPLICATION_REPORTS_DATABASE))

    if (not datamartEnabled):
        # We do not want to proceed if datamart is not enabled.
        return
    if (cfg['ctfcore/dependency/ctfcore-datamart/jdbc/port'] == cfg['ctfcore/dependency/ctfcore-database/jdbc/port']):
        # We do not want to proceed if datamart is running in the same instance as that of prod database
        return
    
    pgsqlDatamartHost = service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL)
    appHost = service.util.isApplicationEnabledLocally(service.constants.APPLICATION_APP)

    for sqlKey in PG_STAT_SQLS_OTHERS:
        if pgsqlDatamartHost:
            cmdStr = "su postgres -c \"psql -p %s -c '%s'\"" % (cfg['ctfcore/dependency/ctfcore-datamart/jdbc/port'], PG_STAT_SQLS_OTHERS[sqlKey])
        elif appHost:
            #TODO: Fix artf148801
            psql_wrapper_path = cfg['RUNTIME_SCRIPTS']
            psql_reporting_wrapper = os.path.join(psql_wrapper_path, 'psql-reporting-wrapper')
            cmdStr = "echo \""+ PG_STAT_SQLS_OTHERS[sqlKey] + "\" | " +  psql_reporting_wrapper
       
        if (pgsqlDatamartHost or appHost):
            log.info("Start of reporting database thread dump of [%s]."%(sqlKey))
            try:
                log.info(app.util.executeCommand(cmdStr))
            except:
                log.exception("Command failed: %s" % cmdStr)

def doCodeSearchDatabaseThreadDump(cfg, log):
    """Dump codesearch database threads"""
    if (checkCodeSearch):
       for sqlKey in PG_STAT_SQLS_OTHERS:
           log.info("Start of codesearch database thread dump of [%s]."%(sqlKey))
           cmdStr = ""
           try:
              cmdStr = """
echo "%s;" \
 | su - bds-codesight -c "%s/CodeSight/postgresql/bin/psql bds_codesight"
    """ %(PG_STAT_SQLS_OTHERS[sqlKey], cfg.get('BDCS_INSTALL_PATH'))
              log.info(app.util.executeCommand(cmdStr))
           except:
              log.exception("Query failed: %s" % cmdStr)

def doSystem(cfg, log):
    """Dump system state into snapshot report."""
    platform = cfg['PLATFORM']
    psCmd = 'ps -efl'
    if platform.lower().find('redhat') > 0:
        psCmd = 'ps auxwwwwwwwf'
    log.info(app.util.executeCommand(psCmd))
    log.info("Running df -kh")
    log.info(app.util.executeCommand('df -kh'))
    log.info("Running top -n 1 b")
    log.info(app.util.executeCommand('top -n 1 b'))
    if platform.lower().find('redhat') > 0:
        log.info("Running free")
        log.info(app.util.executeCommand('free'))
    log.info("Running uptime")
    log.info(app.util.executeCommand('uptime'))
    log.info("Running vmstat 3 3")
    log.info(app.util.executeCommand('vmstat 3 3'))
    log.info("Running w")
    log.info(app.util.executeCommand('w'))
    log.info("Running uname -a")
    log.info(app.util.executeCommand('uname -a'))
    log.info("Running netstat -anp (Ports and Programs)")
    log.info(app.util.executeCommand('netstat -anp'))
    log.info("Running netstat -s (Statistics)")
    log.info(app.util.executeCommand('netstat -s'))
    log.info("Running pstree -u -p")
    log.info(app.util.executeCommand('pstree -u -p'))

def getCurrentTime():
    """Return representation of current time."""
    return "%04d%02d%02d%02d%02d%02d" % time.localtime()[:6]

def snapshot(extra):
    """Take a snapshot of CN and system & write findings to snapshot.log."""
    cfg = app.util.getRuntimeConfiguration()
    label = "snapshot%s" % getCurrentTime()
    logFile = os.path.join(cfg.get('RUNTIME_LOG_DIR'), "snapshot.log")
    log = app.logutil.getNewLogger("Snapshot", logFile, None, None, False,
                    verbose)
    checkCodeSearchEnabled()
    if verbose:
        log.setLevel(logging.DEBUG)
    log.info("BEGIN %s." % label)
    doJVMThreadDump(cfg, log, label)
    doDatabaseThreadDump(cfg, log)
    doReportingDatabaseThreadDump(cfg, log)
    doCodeSearchDatabaseThreadDump(cfg, log)
    doSystem(cfg, log)
    doApache(cfg, log)
    if extra:
        log.info(app.util.executeCommand(extra))
    log.info("END %s." % label)

def checkCodeSearchEnabled():
    appName = service.constants.APPLICATION_CODESEARCH
    global checkCodeSearch
    if (service.util.isApplicationEnabledLocally(appName)):
       checkCodeSearch = True
    else:
       checkCodeSearch = False

def usageAndExit(errCode = 0, errMessage = None):
    """Print usage, error message (if any), and exit.
    """
    sys.stdout.write(__doc__)
    if errMessage:
        sys.stderr.write("ERROR: %s\n" % errMessage)
    sys.exit(errCode)

def main():
    opts, args = getopt.getopt(sys.argv[1:], "he:v", ("help", "extra=", "verbose"))
    extra = None
    global verbose
    for option, value in opts:
        if option in ("-h", "--help"): usageAndExit()
        if option in ("-e", "--extra"):
            extra = value
        if option in ("-v", "--verbose"):
            verbose = True  
    if args: usageAndExit('Unexpected extra args: %s.' % args)
    snapshot(extra)

if __name__ == '__main__':
    main()
