#!/bin/bash
# Utility to fix the filesystem permisssions that can occur if you
# migrate your data and the uid's between machines do not match.

# The script takes one optional numeric argument
# specifying the number of concurrent chown commands to run.

init() {
  # Set shell option to enable dotglob property so that wildcard("*") will
  # match the hidden files(filename begins with ".") too
  shopt -s dotglob

  OPTIONS_FILE="$RUNTIME_DIR"/scripts/options-to-shell-variables.py
  if [ -e "$OPTIONS_FILE" ]; then
     eval "`$OPTIONS_FILE`"
  else
     echo -e "\nThe file $OPTIONS_FILE does not exist\n\nPlease recreate runtime and try again\n"
     exit 1
  fi

  MAIN_numrepos=5
  MAIN_numchowns=`grep processor /proc/cpuinfo | wc -l`
  MAIN_fixall="true"
  MAIN_fixfor=""
}

main() {

  if [ ${MAIN_fixall} == "true" ]; then
     do_fix_app_user_directories
     do_fix_http_user_directories
     do_fix_user_home $HTTPD_USER $HTTPD_GROUP 
     do_fix_user_home $APP_USER $APP_GROUP 
     do_fix_scmviewer_home
     do_fix_svn_repositories
     do_fix_cvs_repositories
     do_fix_git_repositories
     do_fix_cli_data_directories
     do_fix_rb_data_directories
     do_fix_bdb_directories
  else
     for item in ${MAIN_fixfor}
     do

       if [ ${item} == "app" ]; then
          do_fix_app_user_directories
          do_fix_user_home $HTTPD_USER $HTTPD_GROUP
          do_fix_user_home $APP_USER $APP_GROUP
       elif [ ${item} == "http" ]; then
          do_fix_http_user_directories
       elif [ ${item} == "svn" ]; then
          do_fix_svn_repositories
       elif [ ${item} == "cvs" ]; then
          do_fix_cvs_repositories
       elif [ ${item} == "git" ]; then
          do_fix_git_repositories
       elif [ ${item} == "cli" ]; then
          do_fix_cli_data_directories
       elif [ ${item} == "rb" ]; then
          do_fix_rb_data_directories
       elif [ ${item} == "bdb" ]; then
          do_fix_bdb_directories
       else
          echo "WARN: Invalid item [$item]. Skipping .."
       fi
     done
  fi
}

# Fix Git Repositories and Directories
do_fix_git_repositories(){

  echo "NOTICE: Fixing Git Repositories and Git Directories ... "
  USER=$GERRIT_USER
  GROUP="gerrit"
  GIT_REPOSITORY_BASE=$GERRIT_DATA_DIR/gitroot
  GIT_LFS_BASE=$GERRIT_LFS_BASE
  GIT_USER_HOME=/opt/collabnet/gerrit
  GIT_VAR_ETC=$GERRIT_DATA_DIR/etc
  GIT_VAR_SSH=$GERRIT_DATA_DIR/.ssh
  GIT_WARS="/usr/lib/gerrit-ng.war /usr/lib/gerrit-2.1-SNAPSHOT.war"
  GIT_JARS="/usr/lib/teamforge-mappings-migrator.jar /usr/lib/teamforge-migrator.jar"
  if [ ! -z "$GERRIT_USER" ]; then
     DIRS=$GIT_REPOSITORY_BASE
     fix_scm_permissions
     DIRS=$GIT_LFS_BASE
     fix_permissions
     DIRS=$GIT_USER_HOME
     fix_permissions
     DIRS=$GIT_WARS
     fix_permissions
     DIRS=$GIT_JARS
     fix_permissions
     echo "Fixing ownership of links in $GERRIT_DATA_DIR to $USER:$GROUP"
     do_chown "-h" "$GIT_VAR_ETC"
     do_chown "-h" "$GIT_VAR_SSH"
     do_chown "-h" "$GIT_REPOSITORY_BASE"
  fi
}

# Fix CLI Directories
do_fix_cli_data_directories(){

  echo "NOTICE: Fixing CLI Directories ... "
  USER=$HTTPD_USER
  GROUP=$HTTPD_GROUP
  CLI_DATA_HOME=$DATA_DIR/cliserver/cli
  if [ ! -z "$HTTPD_USER" ]; then
     DIRS=$CLI_DATA_HOME
     if [ -d "$CLI_DATA_HOME" ]; then
       fix_permissions
     else 
       echo "Skipping to fix permission on cli data directory. Directory does not exist [$CLI_DATA_HOME]";
     fi
  fi
}
# Fix Review Board Directories
do_fix_rb_data_directories(){

  echo "NOTICE: Fixing Review Board Directories ... "
  USER=$HTTPD_USER
  GROUP=$HTTPD_GROUP
  RB_DATA_HOME=/opt/collabnet/reviewboard/data
  if [ ! -z "$HTTPD_USER" ]; then
     DIRS=$RB_DATA_HOME
     if [ -d "$RB_DATA_HOME" ]; then
       fix_permissions
     else
       echo "Skipping to fix permission on reviewboard data directory. Directory does not exist [$RB_DATA_HOME]";
     fi
  fi
}

# Fix Review Board Directories
do_fix_user_home( ){

  echo "NOTICE: Fixing APP_USER OR HTTP_USER Directories ... "
  USER=$1
  GROUP=$2
  HOME_DIR=/opt/collabnet/teamforge/var/home/$USER
  if [ ! -z "$USER" ]; then
     DIRS=$HOME_DIR
     if [ -d "$HOME_DIR" ]; then
       fix_permissions
     else
       echo "Skipping to fix permission on APP_USER or HTTP_USER directory. Directory does not exist [$HOME_DIR]";
     fi
  fi
}
do_fix_scmviewer_home( ){

  echo "NOTICE: Fixing scmviewer home Directories ... "
  USER=scmviewer
  GROUP=users
  HOME_DIR=/opt/collabnet/teamforge/var/home/s/sc/scmviewer
  if [ ! -z "$USER" ]; then
     DIRS=$HOME_DIR
     if [ -d "$HOME_DIR" ]; then
       fix_permissions
     else
       echo "Skipping to fix permission on scmviewer directory. Directory does not exist [$HOME_DIR]";
     fi
  fi
}


do_fix_bdb_directories() {
  echo "NOTICE: Fixing Teamforge BDB accessible Directories ... "

  BDS_SCAN_DIR=/opt/collabnet/blackduck/scan/
  BDS_POSTGRES_DIR=/opt/collabnet/blackduck/postgres/
  BDS_CODESIGHT_DIR=/opt/collabnet/blackduck/postgres/bds-codesight

  USER=root
  GROUP=root

  if [ -d $BDS_SCAN_DIR ]; then
     echo "Fixing the Ownership of $BDS_SCAN_DIR to $USER:$GROUP"
     chown -RL $USER:$GROUP $BDS_SCAN_DIR
  fi
  
  if [ -d $BDS_POSTGRES_DIR ]; then
     echo "Fixing the Ownership of $BDS_POSTGRES_DIR to $USER:$GROUP"
     chown -L $USER:$GROUP $BDS_POSTGRES_DIR
  fi

  USER=bds-codesight
  GROUP=bds-codesight
  if [ -d $BDS_CODESIGHT_DIR ]; then
     echo "Fixing the Ownership of $BDS_CODESIGHT_DIR to $USER:$GROUP"
     chown -RL $USER:$GROUP $BDS_CODESIGHT_DIR
  fi
}

# Fix CVS Repositories
do_fix_cvs_repositories() {
  echo "NOTICE: Fixing CVS Repositories ... "
  USER='nobody'
  if [ $DATABASE_TYPE == "postgresql" ]; then
      fix_cvs_permissions
  else
      echo "WARNING: Not fixing CVS permissions, as database type is oracle."
  fi
}

# Fix Subversion Repositories
do_fix_svn_repositories() {
  echo "NOTICE: Fixing Subversion Repositories ... "
  USER=$HTTPD_USER
  GROUP=$HTTPD_GROUP
  DIRS=$SUBVERSION_REPOSITORY_BASE
  fix_scm_permissions
  echo "Fixing the Ownership of subdirectories, if any, in $SUBVERSION_BRANDING_REPOSITORY_BASE recursively, to $HTTPD_USER:$HTTPD_GROUP"
  chown -RL $HTTPD_USER:$HTTPD_GROUP "$SUBVERSION_BRANDING_REPOSITORY_BASE"/* &> /dev/null
}

# Fix HTTPD accessible Directories
do_fix_http_user_directories() {
  echo "NOTICE: Fixing Apache accessible Directories ... "
  HTTPD_USER_DIRS="$OVERRIDES_DIR $PUBLISH_DIR \
                 $BDB_LOGFILE_DIR $BDB_DATAFILE_DIR $BDB_ENVFILE_DIR"
  USER=$HTTPD_USER
  GROUP=$HTTPD_GROUP
  DIRS=$HTTPD_USER_DIRS
  fix_permissions
}

# Fix Teamforge app user accessible Directories
do_fix_app_user_directories() {
  echo "NOTICE: Fixing Teamforge App accessible Directories ... "
  APP_USER_DIRS="$FILESTORAGE_LOCATION $DELETED_FILES_LOCATION $DATA_ETC_LOCATION $SCM_REPOSITORY_ARCHIVE $TEMPLATES_DIR \
               $DATA_DIR/events $DATA_DIR/cracklib $AVATAR_SERVICE_DISK_STORE"
  USER=$APP_USER
  GROUP=$APP_GROUP
  DIRS=$APP_USER_DIRS
  fix_permissions

  # sf-svnroot sub-directories are owned by $HTTPD_USER. so dont chown recursively - artf106798
  if [ -d $SUBVERSION_BRANDING_REPOSITORY_BASE ]; then
     echo "Fixing the Ownership of $SUBVERSION_BRANDING_REPOSITORY_BASE to $USER:$GROUP"
     chown -L $USER:$GROUP $SUBVERSION_BRANDING_REPOSITORY_BASE
  fi

  # cvsroot sub-directories are owned by user 'nobody'. so dont chown recursively - artf106798
  if [ -d $CVS_REPOSITORY_BASE ]; then
     echo "Fixing the Ownership of $CVS_REPOSITORY_BASE to $USER:$GROUP"
     chown -L $USER:$GROUP $CVS_REPOSITORY_BASE
  fi
}

fix_cvs_permissions() {
  cvsrecords=`"$RUNTIME_DIR"/scripts/psql-wrapper -c "SELECT sr.id, sr.repository_path from scm_repository sr, external_system es where sr.system_id = es.id and es.adapter_name='CVS'" --tuples-only`
  if [ "$?" -ne "0" ]; then
     echo -e "\n ERROR: This script requires the postgresql service to be running. Please start it and rerun this script \n"
  fi
  IFS=$'\n'
  for record in $cvsrecords
  do
    repo_id=`echo $record | awk '{print $1}'`
    repo_path=`echo $record | awk '{print $3}'`
    grep -i "^$repo_id:" /etc/group > /dev/null
    if [ "$?" != 0 ]; then
       echo "Creating the group $repo_id, as it belongs to the repository $repo_path"
       groupadd $repo_id
    fi
    if [ -d "$repo_path" ]; then
       echo "Fixing the Ownership of $repo_path recursively, to $USER:$repo_id"
       chown -RL $USER:$repo_id $repo_path
    fi
  done
}

fix_scm_permissions() {
  lnumchowns=$MAIN_numchowns
  if [ -z $lnumchowns ]
  then
    numprocs=`grep processor /proc/cpuinfo | wc -l`
    lnumchowns=$numprocs
  fi

  # make sure we have at least 1
  if [ $lnumchowns -lt 1 ]
  then
    lnumchowns=1
  fi
  # increase lnumchowns by 1 to account for the parent process
  lnumchowns=`expr $lnumchowns + 1`

  if [ -z $MAIN_numrepos ]
  then
     MAIN_numrepos=`expr $numprocs \* 10`
  fi

  for location in $DIRS
  do
     echo "Fixing the Ownership of $location recursively, to $USER:$GROUP"
     if [ -d "$location" -a "$(ls -A $location 2> /dev/null | wc -l)" != 0 ]; then
          for repo in $location/*
          do
              jobs=`jobs -p | wc -l`
              while [ $jobs -ge $MAIN_numrepos ]
              do
                  sleep 1
                  jobs=`jobs -p | wc -l`
              done
              (
              if [ -d "$repo" ]
              then
             for subrepo in "$repo"/*
             do
                 if [ "$subrepo" == "$repo/db" ]
                 then
                     for subdb in "$subrepo"/*
                     do
                 	if [ -d "$subdb" -a "$(ls -A "$subdb" 2> /dev/null | wc -l)" != 0 ]
                 	then
                 	    for subsubdb in "$subdb"/*
                 	    do
                 		do_rchown "$subsubdb"
                 	    done
                 	fi
                 	do_lchown "$subdb"
                     done
                     do_lchown "$subrepo"
                 else
                     do_rchown "$subrepo"
                 fi
             done
              fi
              do_lchown "$repo"
              wait
              ) &
          done
     fi
     if [ -d "$location" ]; then
        do_lchown "$location"
     fi
  done
  wait
}

fix_permissions() {
  lnumchowns=$MAIN_numchowns
  if [ -z $lnumchowns ]
  then
     lnumchowns=`grep processor /proc/cpuinfo | wc -l`
     lnumchowns=`expr $lnumchowns \* 10`
  fi
  # make sure we have at least 1
  if [ $lnumchowns -lt 1 ]
  then
     lnumchowns=1
  fi
  # increase lnumchowns by one to account for parent process
  lnumchowns=`expr $lnumchowns + 1`

  for location in $DIRS
  do
      echo "Fixing the Ownership of $location recursively, to $USER:$GROUP"
      if [ -d "$location" -a "$(ls -A $location 2> /dev/null | wc -l)" != 0 ]; then
              for subdir in $location/*
              do
                  do_rchown $subdir
              done
      fi
      if [ -e "$location" ]; then
           do_lchown $location
      fi
  done
  wait
}

do_rchown () {
    do_chown "-LR" "$1"
}

do_lchown () {
    do_chown "" "$1"
}

do_chown () {
    options=$1
    file="$2"
    jobs=`jobs -p | wc -l`
    while [ $jobs -ge $lnumchowns ]
    do
        sleep 1
        jobs=`jobs -p | wc -l`
    done
    chown $options $USER:$GROUP "$file" &
}

is_number() {
  checkarg=$1
  if ! [[ "$checkarg" =~ ^[0-9]+$ ]] ; then
    return 1
  fi

  return 0
}

set_dir_threadnum() {

   inarg=$1
   is_number $inarg

   if [ $? == 0 ]; then
      MAIN_numchowns=$inarg
      echo "Set Repo Processing Thread Count to $MAIN_numchowns"
   else
      echo "ERROR: The value passed to option -d should be integer"
      exit 1
   fi
}

set_repo_threadnum() {

   inarg=$1
   is_number $inarg

   if [ $? == 0 ]; then
      MAIN_numrepos=$inarg
      echo "Set Repo Processing Thread Count to $MAIN_numrepos"
   else
      echo "ERROR: The value passed to option -r should be integer"
      exit 1
   fi
}

set_fix_items() {

   inarg=$1

   if [ ${MAIN_fixall} == "true" ]; then
      MAIN_fixall="false"
   fi
   MAIN_fixfor="$MAIN_fixfor $inarg"
}

show_help() {
   echo
   echo "Usage:"
   echo
   echo "fix_data_permissons.sh [-r <repo_thread_count>] [-d <dir_thread_count>] [-t <fix_permission item>]'"
   echo "Options:"
   echo "   -r  :   Number of repos to process at a time. Default=5"
   echo "   -d  :   Number of directories to process at a time. Default=No of processors"
   echo "   -t  :   Item to Fix. Possible values [app, http, svn, cvs, git]. Default=<all items>"
   echo "   -h  :   Print this help information."
   echo
   echo "Example:"
   echo "   1. To Fix permissions only git repositories, run the below command"
   echo "      ./fix_data_permissons.sh -t git"
   echo "   2. To Fix permissions only gitrepositories and app directories, run the below command"
   echo "      ./fix_data_permissons.sh -t git -t app"
   echo
}

if [ `id -u` -ne 0 ]; then
    echo "You must run this script as root!"
    exit 1
fi

## Initialise Runtime Variables
init

## Processing the command arguments
while getopts ":r:d:t:h" opt; do
   case $opt in
      r)
        set_repo_threadnum $OPTARG
        ;;
      d)
        set_dir_threadnum $OPTARG
        ;;
      t)
        set_fix_items $OPTARG
        ;;
      h)
        show_help
        exit 0
        ;;
      \?)
        echo "ERROR: Invalid option: -$OPTARG"
        show_help
        exit 1
        ;;
      :)
        echo "ERROR: Option -$OPTARG requires an argument."
        exit 1
        ;;
   esac
done

## Calling the main function that runs respective fix permission
## based on "-t" option.
main

wait
