#!/usr/bin/env python2

__copyright__ = "Copyright (C) 2008-2013 CollabNet, Inc."
__license__ = "Commercial/Restrictive"

import os
import re
import sys
import getopt
import getpass
import datetime
import app.util
import app.defaults
import app.tokens.defines
import app.options

def printUsageAndExit(error=None, exit_code=0):
    """ Prints Help content about this program and exits """

    print
    print "This script encrypts or decrypts the given site option file."
    print
    print "Valid options: "
    print "  -h | --help         print this usage message and exit"
    print "  -e | --encrypt      encrypt the site option file. (default operation)"
    print "  -d | --decrypt      decrypt the site option file."
    print "  -i | --infile=      path of the input site option file that needs to be "
    print "                      encrypted/decrypted. (will be prompted if not supplied)"
    print "  -o | --outfile=     path of the output site option file that will be "
    print "                      encrypted/decrypted. assumes infile by default if not supplied. "
    print
    print "Important note: If arguments are not specified, then they are prompted"
    print "                to user through stdin."
    print
    if error:
        print >> sys.stderr, error
    sys.exit(exit_code)

def printLog(logMsg, printTime=True):
    """ Prints program output messages both to logfile and tty """

    global MAIN_logfile
    try:
       logFh = open (MAIN_logfile, "a")
    except:
       # If unable to write to Gerrit's log directory, Creating in /tmp
       MAIN_logfile = "/tmp/soconf.log"
       logFh = open (MAIN_logfile, "a")

    if printTime:
       now = datetime.datetime.now()
       curtime  = now.strftime("%d-%m-%Y %H:%M:%S")
       logFh.write(curtime + ": " + logMsg)
    else:
       logFh.write(logMsg)
    logFh.close()
    sys.stdout.write(logMsg)
    sys.stdout.flush()

def parseConfigFile(confFile):
    """ Parses the given config file as key-value pair and returns a dict structure """

    readFh = open(confFile, "r")
    confLines = readFh.read().split("\n")
    readFh.close()

    matcher = '^\s*([\w\.\-]+)\s*=\s*(.*)\s*$'
    result  = None
    rtcfg   = {}
    for rec in confLines:
        mObj = re.search(r'%s'%matcher, rec)
        if mObj == None:
            continue
        key = mObj.group(1)
        val = mObj.group(2)
        rtcfg[key] = val

    if not rtcfg.has_key('OBFUSCATION_KEY'):
       rtcfg['OBFUSCATION_KEY'] = app.defaults.DEFAULT_OBFUSCATION_KEY

    if not rtcfg.has_key('OBFUSCATION_PREFIX'):
       rtcfg['OBFUSCATION_PREFIX'] = app.defaults.DEFAULT_OBFUSCATION_PREFIX

    if not rtcfg.has_key('OBFUSCATION_ENABLED'):
       rtcfg['OBFUSCATION_ENABLED'] = app.defaults.DEFAULT_OBFUSCATION_ENABLED
    return rtcfg

def initialiseGlobals():
    """ Intializes the global variables """

    global MAIN_cfg, MAIN_tkn, MAIN_oper, MAIN_infile, MAIN_outfile
    global MAIN_logfile, MAIN_rtfile

    MAIN_cfg  = app.util.getRuntimeConfiguration()
    MAIN_tkn  = app.tokens.defines.TokenDef()
    MAIN_oper = 'encrypt'
    MAIN_infile = None
    MAIN_outfile = None

    MAIN_logfile = os.path.join(MAIN_cfg['RUNTIME_LOG_DIR'], "soconf.log")
    MAIN_rtfile  = os.path.join(MAIN_cfg['RUNTIME_DIR'], "conf", "runtime-options.conf")

def checkEnvironment():
    """ Checks if this program can execute. Exits with reason if check fails """

    global MAIN_cfg, MAIN_infile, MAIN_outfile

    if (getpass.getuser() != 'root'):
       printLog("Error: This program should run as root user.\n")
       sys.exit(1)

    printLog("Checking if runtime is in Obfuscation mode ... ")
    if (MAIN_cfg['OBFUSCATION_ENABLED'] != 'true'):
       printLog("Error: Teamforge is not running in obfuscation mode.\n")
       printLog("Reason: This program runs only if Teamforge is configured in obfuscation mode.\n")
       printLog("        Token OBFUSCATION_ENABLED is not set to 'true'.\n")
       sys.exit(1)

    if (MAIN_outfile == None):
       MAIN_outfile = MAIN_infile
    else:
       outDir = os.path.dirname(MAIN_outfile)
       if (not os.path.exists(outDir)):
          printLog("Error: The path to output file [%s] is not valid.\n"%(MAIN_outfile))
          printLog("Reason: The directory path %s may not exists! Provide a valid file path.\n"%(outDir))
          sys.exit(1)
    printLog("Done\n", False)

    printLog("Verifying obfusction environment in runtime and site option files ... ")
    rtcfg = parseConfigFile(MAIN_infile)

    if (MAIN_cfg['OBFUSCATION_ENABLED'] != rtcfg['OBFUSCATION_ENABLED']):
       printLog("\n")
       printLog("Error: Obfuscation environment mismatch!\n")
       printLog("Reason: Your runtime has OBFUSCATION_ENABLED => %s, whereas your siteoption has OBFUSCATION_ENABLED => %s\n" \
           %(MAIN_cfg['OBFUSCATION_ENABLED'], rtcfg['OBFUSCATION_ENABLED']))
       printLog("Fix: Set OBFUSCATION_ENABLED=%s in your site options file and re-try.\n"%(MAIN_cfg['OBFUSCATION_ENABLED']))
       sys.exit(1)

    if (MAIN_cfg['OBFUSCATION_KEY'] != rtcfg['OBFUSCATION_KEY']):
       printLog("\n")
       printLog("Error: Obfuscation environment mismatch!\n")
       printLog("Reason: Your runtime has OBFUSCATION_KEY => %s, whereas your siteoption has OBFUSCATION_KEY => %s\n" \
           %(MAIN_cfg['OBFUSCATION_KEY'], rtcfg['OBFUSCATION_KEY']))
       printLog("Fix: Set OBFUSCATION_KEY=%s in your site options file and re-try.\n"%(MAIN_cfg['OBFUSCATION_KEY']))
       sys.exit(1)

    if (MAIN_cfg['OBFUSCATION_PREFIX'] != rtcfg['OBFUSCATION_PREFIX']):
       printLog("\n")
       printLog("Error: Obfuscation environment mismatch!\n")
       printLog("Reason: Your runtime has OBFUSCATION_PREFIX => %s, whereas your siteoption has OBFUSCATION_PREFIX => %s\n" \
           %(MAIN_cfg['OBFUSCATION_PREFIX'], rtcfg['OBFUSCATION_PREFIX']))
       printLog("Fix: Set OBFUSCATION_PREFIX=%s in your site options file and re-try.\n"%(MAIN_cfg['OBFUSCATION_PREFIX']))
       sys.exit(1)

    printLog("Done\n", False)

def writeObfResponse(outfile, fileBuffer):
    """ Writes the obfuscation response file buffer to a outfile """

    printLog ("Saving to output file [%s].... "%(outfile))
    fileData = "\n".join(fileBuffer)
    writeFh = open(outfile, "w")
    writeFh.write(fileData)
    writeFh.close()
    printLog ("Done\n", False)

def processObfRequest(oper, infile):
    """ Converts (encrypt/decrypt)s the infile based on oper """

    printLog ("Loading input file [%s].... "%(infile))
    readFh = open(infile, "r")
    confLines = readFh.read().split("\n")
    readFh.close()
    printLog ("Done\n", False);

    matcher = '^\s*([\w\.\-]+)\s*=\s*(.*)\s*$'
    result  = None
    outBuffer = []
    printLog ("Conversion in progress .")
    for rec in confLines:
        mObj = re.search(r'%s'%matcher, rec)
        if mObj == None:
           outBuffer.append(rec)
           continue
        key = mObj.group(1)
        val = mObj.group(2)
        if (MAIN_tkn.getProperty(key, 'obfuscate') == "true"):
           printLog(".", False)

           obfuscator = app.options.Obfuscator.createFromRuntimeOptions(MAIN_cfg)

           if (oper == 'encrypt'):
              val = obfuscator.obfuscate(val)
           else:
              val = obfuscator.deobfuscate(val)
        outBuffer.append("%s=%s"%(key,val))
    printLog(" Done\n", False)

    return outBuffer

if __name__ == '__main__':

   initialiseGlobals()
   try:
      (opts, args) = getopt.getopt (sys.argv[1:], "hcedi:o:",
                             [ "help", "convert", "encrypt", "decrypt", "infile=", "outfile=" ])
   except:
       printUsageAndExit("Invalid options specified!!", 1)

   for option, value in opts:
       if option in ("-h", "-help"):
          printUsageAndExit()
       elif option in ("-e", "--encrypt"):
          MAIN_oper = 'encrypt'
       elif option in ("-d", "--decrypt"):
          MAIN_oper = 'decrypt'
       elif option in ("-i", "--infile"):
          MAIN_infile = value
       elif option in ("-o", "--outfile"):
          MAIN_outfile = value
       elif option in ("-c", "--convert"):
          pass

   if MAIN_infile == None:
      while True:
         MAIN_infile = raw_input ("Enter Site option filename to %s: "%(MAIN_oper))
         if (len(MAIN_infile) == 0):
            print "Site option filename cannot be empty. Please re-try."
            continue
         elif (not os.path.isfile(MAIN_infile)):
            print "Site option filename [%s] is not a valid file or path. Please re-try."%(MAIN_infile)
            continue
         else:
            break

   checkEnvironment()
   outBuffer = processObfRequest(MAIN_oper, MAIN_infile)
   writeObfResponse(MAIN_outfile, outBuffer)
