import os
import glob
import time
import signal

import app.servicecontrol
import app.util
import app.defaults

START_TIMEOUT = 30

class ServiceController(app.servicecontrol.ServiceController):
    def __init__(self, service, logger):
        super(ServiceController, self).__init__(service, logger)
        
        self.__lastStartupTime = 0
    
    def getPidFile(self):
        port = self.getService().getProvidedEndpoint("jdbc").getPort()
        
        # Note: If there are multiple Postgres instances running in parallel,
        # we need to take the PID that corresponds to the instance's port number.
        
        pidFiles = glob.glob("/var/run/postmaster*.%d.pid" % port)
        if pidFiles:
            return pidFiles[0]
        else:
            return None
            
    def getKillSignal(self):
        # Postgres really, really does not like SIGKILLs - it will leave 
        # child processes behind and may corrupt data. Therefore, be
        # a little more gentle.
        return signal.SIGQUIT
            
    def __isRunningUncontrolledWithoutDeployment(self):
        # If somebody wiped the runtime without stopping Postgres first,
        # we are in an odd state.
        return self.isProcessRunning() and \
            not super(ServiceController, self).isDeployed() and \
            not os.path.isfile(app.defaults.RUNTIME_OPTIONS_CONF)
    
    def isDeployed(self):
        if self.__isRunningUncontrolledWithoutDeployment():
            return True
        else:
            return super(ServiceController, self).isDeployed()
            
    def isDoomed(self):
        if self.__isRunningUncontrolledWithoutDeployment():
            return True
        else:
            return super(ServiceController, self).isDoomed()
    
    def canBootstrap(self):
        # Bootstrapping is a no-op, so allow it always.
        return True
        
    def __runInit(self, verb):
        config = app.util.getRuntimeConfiguration()

        logFile = os.path.join(config['PGSQL_LOG_DIR'], 'service.log')
        initScript = config["PGSQL_SERVICE_CMD"]

        return os.system("%s %s < /dev/null >> %s 2>&1 &" % (initScript, verb, logFile)) == 0
        
    def isStarting(self):
        # Postgres does not immediately publish its PID file - but that does not mean
        # it is in trouble. So we need to give it some extra time before making any status
        # decisions.
        return not self.getPid() and (time.time() - self.__lastStartupTime) < START_TIMEOUT
        
    def startDaemonAsync(self):
        if self.__runInit("start"):
            self.__lastStartupTime = time.time()
            return True
        else:
            return False
        
    def stopDaemonAsync(self):
        return self.__runInit("stop")
        
    def createPostDeployAction(self):
        # Start the service right after deployment
        return self.createStartAction()
