#!/usr/bin/env python

""" Mount/UnMount App Server's Filestorage in Search Server through NFS """

import sys
import getopt
import os
import shutil
import commands

import app.util
import service.util
import service.constants

cfg = app.util.getRuntimeConfiguration()

def usage(errorMessage=None):
    """ Prints usage details """

    if errorMessage:
        stream = sys.stderr
    else:
        stream = sys.stdout

    stream.write("Usage: \n"
                 "  %s [--help] | [--mount] | [--umount]\n"
                 % (sys.argv[0],))

    stream.write("\n"
                 "Options: \n"
                 "  -m, --mount              mount's appserver's filestorage\n"
                 "  -u, --umount             unmount's appserver's filestorage\n"
                 "  -h, --help               print this help message\n"
                 "\n"
                 "Description: \n"
                 "  * This script mounts / unmounts app server's filestorage into index server\n"
                 "    using nfs when app and indexer services are running on a separate box.\n"
                 "  * This script is called by search.sh script during its start and stop operation.\n"
		 "  * This script will run only in indexer server box and opererates only if app\n"
                 "    server is running remotely\n"
		 "  * This script is not intended to run manually or without knowledge on Teamforge\n"
                 "    system setup.\n")

    if errorMessage:
        stream.write("\nERROR:%s\n" % errorMessage)
        exit_code = 1
    else:
        exit_code = 0

    sys.exit(exit_code)


def main():
    opts, args = getopt.getopt(sys.argv[1:], 'hmu',
                               ['help', 'mount', 'umount'])

    # Default operation is "mount" if no options are provided
    operation = "mount"

    for opt, value in opts:
        if (opt in ('-h', '--help')):
            usage()
        elif (opt in ('-m', '--mount')):
            operation = "mount"
        elif (opt in ('-u', '--umount')):
            operation = "umount"
        else:
            usage('Unknown Option : ' + opt)

    # Exit if target box is any of the below
    # 1. Indexer Service is not running locally
    # 2. Indexer and App are running in same box
    _exit_If_Not_Runnable()

    if (operation == "mount"):
       handleMount()
    elif (operation == "umount"):
       handleUnMount()

def _exit_If_Not_Runnable():

    if not service.util.isApplicationEnabledLocally(service.constants.APPLICATION_INDEXER):
       print "Indexer Service is not running locally. Skipping.\n"
       sys.exit(0)

    if (service.util.isApplicationEnabledLocally(service.constants.APPLICATION_INDEXER) and
        service.util.isApplicationEnabledLocally(service.constants.APPLICATION_APP)):
        print "Indexer and App Service are running locally. Skipping.\n"
        sys.exit(0)

    # Exit if INDEXER_NFS_FILESTORAGE_LOCATION is not a nfs path.
    # For SAAS, most of the time, OPS manage NFS mount by setting entries in /etc/fstab.
    # In such cases, Teamforge need not mount the filestorage, as both mount and unmount
    # are managed by OPS.
    try:
        srcLocation = cfg.get('INDEXER_NFS_FILESTORAGE_LOCATION')
        srcLocation.index(':')
    except:
        print "NFS Filestorage Location could be automounted/managed. Skipping.\n"
        sys.exit(0)
       

def _ifMounted(chkLocation):
    """ Returns True/False if "chkLocation" is already mounted (nfs type mount) """

    chkLocation = os.path.realpath(chkLocation)
    mountCmd = "mount -t nfs -l | grep 'on %s'"%(chkLocation)
    (ret, output) = commands.getstatusoutput(mountCmd)

    if (ret == 0):
       return True

    return False

def _unmountNFS(mountLocation):
    """ Does unmounting of "mountLocation" """

    mountLocation = os.path.realpath(mountLocation)
    mountCmd = "umount -t nfs %s"%(mountLocation)
    (ret, output) = commands.getstatusoutput(mountCmd)
    return ret

def _mountNFS(srcLocation, dstLocation):
    """ Does nfs mount of remote "srcLocation" on host "srcHost" locally at "dstLocation" """

    mountCmd = "mount -t nfs %s %s -r"%(srcLocation, dstLocation)
    (ret, output) = commands.getstatusoutput(mountCmd)

    if (ret != 0):
      print "ERROR: Unable to mount remote file system\n";
      print "REASON: %s\n"%(output);

    return ret

def handleMount():
    """ Handles Mount operations """

    srcLocation = cfg.get('INDEXER_NFS_FILESTORAGE_LOCATION')
    srcLocation = os.path.normpath(srcLocation)
    fsLocation = cfg.get('FILESTORAGE_LOCATION')
    fsLocation = os.path.normpath(fsLocation)

    if (_ifMounted(fsLocation)):
       _unmountNFS(fsLocation)

    ret = _mountNFS(srcLocation, fsLocation)

    if (ret != 0):
       print "NFS Mount of %s to %s failed."%(srcLocation, fsLocation)
       sys.exit(1)

    sys.exit(0)

def handleUnMount():
    """ Handles Unmount operations """

    fsLocation = cfg.get('FILESTORAGE_LOCATION')
    fsLocation = os.path.normpath(fsLocation)

    if not (_ifMounted(fsLocation)):
       sys.exit(0)

    ret = _unmountNFS(fsLocation)
    if (ret != 0):
       print "NFS Unmount of location %s failed."%(fsLocation)
       sys.exit(1)

    sys.exit(0)

if __name__ == '__main__':
    main()
